@@ -30,10 +30,10 @@ FREI0R_REVISION=
 ENABLE_MOVIT=1
 SUBDIRS=
 MOVIT_HEAD=0
-MOVIT_REVISION="origin/shotcut"
+MOVIT_REVISION="origin/shotcut-opengl3"
 ENABLE_SWH_PLUGINS=0
 FFMPEG_HEAD=0
-FFMPEG_REVISION="origin/release/5.1"
+FFMPEG_REVISION="origin/release/6.1"
 FFMPEG_ADDITIONAL_OPTIONS=
 MLT_HEAD=1
 MLT_REVISION=
@@ -48,16 +48,19 @@ ENABLE_ZIMG=1
 ZIMG_HEAD=0
 ZIMG_REVISION="51c3c7f750c2af61955377faad56e3ba1b03589f"
 DAV1D_HEAD=0
-DAV1D_REVISION="1.0.0"
+DAV1D_REVISION="1.3.0"
 AOM_HEAD=0
-AOM_REVISION="v3.4.0"
+AOM_REVISION="v3.8.0"
 ENABLE_VMAF=1
 VMAF_HEAD=0
 VMAF_REVISION="v2.3.1"
 ENABLE_GLAXNIMATE=1
 GLAXNIMATE_HEAD=0
-GLAXNIMATE_REVISION="v0.5.1"
+GLAXNIMATE_REVISION="origin/v0.5.4"
 ENABLE_GOPRO2GPX=1
+ENABLE_OPENCV=1
+OPENCV_HEAD=0
+OPENCV_REVISION="4.7.0"
 
 # QT_INCLUDE_DIR="$(pkg-config --variable=prefix QtCore)/include"
 QT_INCLUDE_DIR=${QTDIR:+${QTDIR}/include}
@@ -177,6 +180,12 @@ function to_key {
     gopro2gpx)
       echo 14
     ;;
+    opencv)
+      echo 15
+    ;;
+    opencv_contrib)
+      echo 16
+    ;;
     *)
       echo UNKNOWN
     ;;
@@ -368,6 +377,9 @@ function set_globals {
     if test "$ENABLE_GOPRO2GPX" = 1 ; then
         SUBDIRS="$SUBDIRS gopro2gpx"
     fi
+    if test "$ENABLE_OPENCV" = 1 ; then
+        SUBDIRS="opencv opencv_contrib $SUBDIRS"
+    fi
     SUBDIRS="$SUBDIRS mlt shotcut"
   fi
 
@@ -408,6 +420,8 @@ function set_globals {
   REPOLOCS[12]="https://github.com/Netflix/vmaf.git"
   REPOLOCS[13]="https://gitlab.com/ddennedy/glaxnimate.git"
   REPOLOCS[14]="https://github.com/ddennedy/gopro2gpx.git"
+  REPOLOCS[15]="https://github.com/opencv/opencv.git"
+  REPOLOCS[16]="https://github.com/opencv/opencv_contrib.git"
 
   # REPOTYPE Array holds the repo types. (Yes, this might be redundant, but easy for me)
   REPOTYPES[0]="git"
@@ -425,6 +439,8 @@ function set_globals {
   REPOTYPES[12]="git"
   REPOTYPES[13]="git"
   REPOTYPES[14]="git"
+  REPOTYPES[15]="git"
+  REPOTYPES[16]="git"
 
   # And, set up the revisions
   REVISIONS[0]=""
@@ -448,7 +464,7 @@ function set_globals {
     REVISIONS[4]="$SHOTCUT_REVISION"
   fi
   REVISIONS[5]="swh-plugins-0.4.15+1"
-  REVISIONS[6]="sdk/8.1" # nv-codec-headers
+  REVISIONS[6]="sdk/12.0" # nv-codec-headers
   REVISIONS[7]="" # AMF
   REVISIONS[8]=""
   if test 0 = "$BIGSH0T_HEAD" -a "$BIGSH0T_REVISION" ; then
@@ -475,6 +491,14 @@ function set_globals {
     REVISIONS[13]="$GLAXNIMATE_REVISION"
   fi
   REVISIONS[14]=""
+  REVISIONS[15]=""
+  if test 0 = "$OPENCV_HEAD" -a "$OPENCV_REVISION" ; then
+    REVISIONS[15]="$OPENCV_REVISION"
+  fi
+  REVISIONS[16]=""
+  if test 0 = "$OPENCV_HEAD" -a "$OPENCV_REVISION" ; then
+    REVISIONS[16]="$OPENCV_REVISION"
+  fi
 
   # Figure out the number of cores in the system. Used both by make and startup script
   CPUS=$(nproc)
@@ -496,7 +520,7 @@ function set_globals {
 
   # set global environment for all jobs
   alias make=mingw32-make
-  export QTDIR="$HOME/Qt/5.15.2/mingw81_64"
+  export QTDIR="$HOME/Qt/6.4.3/mingw_64"
   export PKG_CONFIG_PATH="$HOME/lib/pkgconfig:$PKG_CONFIG_PATH"
   export PATH="$FINAL_INSTALL_DIR/bin:$PATH"
   export LD_RUN_PATH="$FINAL_INSTALL_DIR/lib"
@@ -509,7 +533,7 @@ function set_globals {
   #####
   # ffmpeg
   CONFIG[0]="./configure --prefix=$FINAL_INSTALL_DIR --disable-static --disable-doc --enable-gpl --enable-version3 --enable-shared --enable-runtime-cpudetect $CONFIGURE_DEBUG_FLAG"
-  CONFIG[0]="${CONFIG[0]} --enable-libtheora --enable-libvorbis --enable-libmp3lame --enable-libx264 --enable-libx265 --enable-libvpx --enable-libopus --enable-libmfx --enable-libdav1d --enable-libaom --disable-decoder=libaom_av1 --enable-libwebp"
+  CONFIG[0]="${CONFIG[0]} --enable-libtheora --enable-libvorbis --enable-libmp3lame --enable-libx264 --enable-libx265 --enable-libvpx --enable-libopus --enable-libvpl --enable-libdav1d --enable-libaom --disable-decoder=libaom_av1 --enable-libwebp --disable-vulkan --disable-vaapi"
   # Add optional parameters
   if [ "$ENABLE_ZIMG" = "1" ]; then
     CONFIG[0]="${CONFIG[0]} --enable-libzimg"
@@ -522,29 +546,43 @@ function set_globals {
 
   #####
   # mlt
-  CONFIG[1]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -DGPL=ON -DGPL3=ON -DMOD_GLAXNIMATE=ON -DMOD_GDK=OFF -DMOD_SDL1=OFF $CMAKE_DEBUG_FLAG"
+  CONFIG[1]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -DMOD_GDK=OFF -DMOD_GLAXNIMATE_QT6=ON -DMOD_QT=OFF -DMOD_QT6=ON -DMOD_SDL1=OFF"
+  [ "$ENABLE_OPENCV" = "1" ] && CONFIG[1]="${CONFIG[1]} -DMOD_OPENCV=ON"
+  if [ "$DEBUG_BUILD" = "1" ]; then
+    CONFIG[1]="${CONFIG[1]} -DCMAKE_BUILD_TYPE=RelWithDebInfo"
+  else
+    CONFIG[1]="${CONFIG[1]} -DCMAKE_BUILD_TYPE=Release"
+  fi
   CFLAGS_[1]="-I$FINAL_INSTALL_DIR/include $ASAN_CFLAGS $CFLAGS"
+  CXXFLAGS_[1]="${CFLAGS_[1]} -std=c++11 -D_XOPEN_SOURCE=700"
   LDFLAGS_[1]="-L$FINAL_INSTALL_DIR/lib $ASAN_LDFLAGS $LDFLAGS"
+  BUILD[1]="ninja -j $MAKEJ"
+  INSTALL[1]="ninja install"
 
   #####
   # frei0r
   CONFIG[2]="cmake -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DWITHOUT_GAVL=1 -DWITHOUT_OPENCV=1 -GNinja $CMAKE_DEBUG_FLAG"
   CFLAGS_[2]="$CFLAGS"
   LDFLAGS_[2]=$LDFLAGS
+  BUILD[2]="ninja -j $MAKEJ"
+  INSTALL[2]="ninja install"
 
   #####
   # movit
   CONFIG[3]="./autogen.sh --prefix=$FINAL_INSTALL_DIR"
   # MinGW does not provide ffs(), but there is a gcc intrinsic for it.
-  CFLAGS_[3]="$CFLAGS -Dffs=__builtin_ffs"
-  CFLAGS_[3]="${CFLAGS_[3]} -fpermissive"
+  CFLAGS_[3]="$CFLAGS -Dffs=__builtin_ffs -fpermissive"
+  CXXFLAGS_[3]="${CFLAGS_[3]}"
   LDFLAGS_[3]=$LDFLAGS
+  BUILD[3]="make -j$MAKEJ libmovit.la"
 
   #####
   # shotcut
   CONFIG[4]="cmake -G Ninja -D CMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -D SHOTCUT_VERSION=$SHOTCUT_VERSION $CMAKE_DEBUG_FLAG"
   CFLAGS_[4]="$ASAN_CFLAGS $CFLAGS"
   LDFLAGS_[4]="$ASAN_LDFLAGS $LDFLAGS"
+  BUILD[4]="ninja -j $MAKEJ"
+  INSTALL[4]="install_shotcut"
 
   #####
   # swh-plugins
@@ -559,15 +597,19 @@ function set_globals {
   #######
   # AMF - no build required
   CONFIG[7]=""
+  [ ! -d "$FINAL_INSTALL_DIR/include/AMF" ] && INSTALL[7]="install_amf"
 
   #########
   # bigsh0t
   CONFIG[8]="cmake -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -GNinja $CMAKE_DEBUG_FLAG"
   CFLAGS_[8]=$CFLAGS
   LDFLAGS_[8]=$LDFLAGS
+  BUILD[8]="ninja -j $MAKEJ"
+  INSTALL[8]="install -p -c *.dll "$FINAL_INSTALL_DIR"/lib/frei0r-1"
 
   #####
   # zimg
+  [ ! -e "$SOURCE_DIR"/zimg/configure ] && PRECONFIG[9]="./autogen.sh"
   CONFIG[9]="./configure --prefix=$FINAL_INSTALL_DIR"
   CFLAGS_[9]=$CFLAGS
   LDFLAGS_[9]=$LDFLAGS
@@ -582,6 +624,8 @@ function set_globals {
   fi
   CFLAGS_[10]=$CFLAGS
   LDFLAGS_[10]=$LDFLAGS
+  BUILD[10]="build_dav1d"
+  INSTALL[10]="meson install -C builddir"
 
   #####
   # aom
@@ -589,6 +633,8 @@ function set_globals {
   CONFIG[11]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR $CMAKE_DEBUG_FLAG -DBUILD_SHARED_LIBS=1 -DCONFIG_AV1_DECODER=0 -DENABLE_EXAMPLES=0 -DENABLE_TESTS=0 ../aom"
   CFLAGS_[11]=$CFLAGS
   LDFLAGS_[11]=$LDFLAGS
+  BUILD[11]="ninja -j $MAKEJ"
+  INSTALL[11]="ninja install"
 
   #####
   # vmaf
@@ -600,6 +646,8 @@ function set_globals {
   fi
   CFLAGS_[12]=$CFLAGS
   LDFLAGS_[12]=$LDFLAGS
+  BUILD[12]="ninja -C libvmaf/build -j $MAKEJ"
+  INSTALL[12]="install_vmaf"
 
   #####
   # glaxnimate
@@ -611,12 +659,50 @@ function set_globals {
   fi
   CFLAGS_[13]="$ASAN_CFLAGS $CFLAGS"
   LDFLAGS_[13]="$ASAN_LDFLAGS $LDFLAGS"
+  BUILD[13]="ninja -j $MAKEJ"
+  INSTALL[13]="ninja translations install"
 
   #####
   # gopro2gpx
   CONFIG[14]="cmake -G Ninja -D CMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR $CMAKE_DEBUG_FLAG"
   CFLAGS_[14]="$CFLAGS"
   LDFLAGS_[14]="$LDFLAGS"
+  BUILD[14]="ninja -j $MAKEJ"
+  INSTALL[14]="install -p -c gopro2gpx $FINAL_INSTALL_DIR"
+
+  #####
+  # opencv
+  CONFIG[15]="cmake -B build -G Ninja -D CMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -D BUILD_LIST=tracking -D OPENCV_GENERATE_PKGCONFIG=YES -D OPENCV_EXTRA_MODULES_PATH=../opencv_contrib/modules -D WITH_OPENMP=ON $CMAKE_DEBUG_FLAG"
+  CFLAGS_[15]="$CFLAGS"
+  LDFLAGS_[15]="$LDFLAGS"
+  BUILD[15]="ninja -C build -j $MAKEJ"
+  INSTALL[15]="ninja -C build install"
+}
+
+function build_dav1d {
+  # dav1d frequently fails on Windows on generate symbol file
+  cmd ninja -C builddir -j $MAKEJ || cmd ninja -C builddir -j $MAKEJ
+}
+
+function install_amf {
+  cmd rm -rf Thirdparty
+  cmd mkdir -p "$FINAL_INSTALL_DIR/include/AMF"
+  cmd cp -av "amf/public/include/." "$FINAL_INSTALL_DIR/include/AMF"
+}
+
+function install_shotcut {
+  cmd ninja install
+  cmd install -c COPYING "$FINAL_INSTALL_DIR"
+  cmd install -c packaging/windows/shotcut.iss "$FINAL_INSTALL_DIR"/..
+  cmd sed -i "s/YY.MM.DD/$SHOTCUT_VERSION/" "$FINAL_INSTALL_DIR"/../shotcut.iss
+  cmd install -d "$FINAL_INSTALL_DIR"/share/translations
+  cmd install -p -c translations/*.qm "$FINAL_INSTALL_DIR"/share/translations
+}
+
+function install_vmaf {
+  cmd ninja install -C libvmaf/build || die "Unable to install $1"
+  cmd install -d "$FINAL_INSTALL_DIR"/share/vmaf
+  cmd install -p -c model/*.json "$FINAL_INSTALL_DIR"/share/vmaf || die "Unable to install $1"
 }
 
 ######################################################################
@@ -769,6 +855,7 @@ function get_subproject {
           cmd git --no-pager clone --quiet --recurse-submodules $REPOLOC || die "Unable to git clone source for $1 from $REPOLOC"
           cmd cd $1 || die "Unable to change to directory $1"
           cmd git checkout --recurse-submodules $REVISION || die "Unable to git checkout $REVISION"
+          [ "$SDK" = "1" -a "shotcut" != "$1" -a "mlt" != "$1" ] && cmd rm -rf .git
       fi
   elif test "svn" = "$REPOTYPE" ; then
       # Create subdir if not exist
@@ -883,40 +970,25 @@ function configure_compile_install_subproject {
   log PKG_CONFIG_PATH=$PKG_CONFIG_PATH
   export CFLAGS=`lookup CFLAGS_ $1`
   log CFLAGS=$CFLAGS
+  export CXXFLAGS=`lookup CXXFLAGS_ $1`
+  log CXXFLAGS=$CFLAGS
   export LDFLAGS=`lookup LDFLAGS_ $1`
   log LDFLAGS=$LDFLAGS
 
+  MYPRECONFIG=`lookup PRECONFIG $1`
+  MYCONFIG=`lookup CONFIG $1`
+  MYBUILD=`lookup BUILD $1`
+  MYINSTALL=`lookup INSTALL $1`
+
   # Configure
   if [ "$ACTION_CONFIGURE" = "1" ]; then
 
   # Configure
   feedback_status Configuring $1
 
-  # Special hack for mlt
-  if test "mlt" = "$1"; then
-    export CXXFLAGS="$CFLAGS -std=c++11 -D_XOPEN_SOURCE=700"
-  fi
-
-  # Special hack for movit
-  if test "movit" = "$1"; then
-#    export PATH="$HOME/mingw810_64/bin:$PATH"
-    export CXXFLAGS="$CFLAGS"
-  fi
-
-  # Special hack for AMF
-  if test "AMF" = "$1" -a ! -d "$FINAL_INSTALL_DIR/include/AMF"; then
-    cmd rm -rf Thirdparty
-    cmd mkdir -p "$FINAL_INSTALL_DIR/include/AMF"
-    cmd cp -av "amf/public/include/." "$FINAL_INSTALL_DIR/include/AMF"
-  fi
-
-  # Special hack for zimg
-  if test "zimg" = "$1" -a ! -e configure ; then
-    debug "Need to create configure for $1"
-    cmd ./autogen.sh || die "Unable to create configure file for $1"
-    if test ! -e configure ; then
-      die "Unable to confirm presence of configure file for $1"
-    fi
+  if test "$MYPRECONFIG" != ""; then
+    cmd $MYPRECONFIG || die "Unable to pre-configure $1"
+    feedback_status Done pre-configuring $1
   fi
 
   # Special hack for aom
@@ -925,7 +997,6 @@ function configure_compile_install_subproject {
     cmd cd ../build-aom || die "Unable to change to directory aom/builddir"
   fi
 
-  MYCONFIG=`lookup CONFIG $1`
   if test "$MYCONFIG" != ""; then
     cmd $MYCONFIG || die "Unable to configure $1"
     feedback_status Done configuring $1
@@ -935,14 +1006,8 @@ function configure_compile_install_subproject {
 
   # Compile
   feedback_status Building $1 - this could take some time
-  if test "movit" = "$1" ; then
-    cmd make -j$MAKEJ libmovit.la || die "Unable to build $1"
-  elif test "frei0r" = "$1" -o "bigsh0t" = "$1" -o "aom" = "$1" -o "mlt" = "$1" -o "shotcut" = "$1" -o "glaxnimate" = "$1" -o "gopro2gpx" = "$1"; then
-    cmd ninja -j $MAKEJ || die "Unable to build $1"
-  elif test "dav1d" = "$1"; then
-    cmd ninja -C builddir -j $MAKEJ || die "Unable to build $1"
-  elif test "vmaf" = "$1"; then
-    cmd ninja -C libvmaf/build -j $MAKEJ || die "Unable to build $1"
+  if test "$MYBUILD" != ""; then
+    cmd $MYBUILD || die "Unable to build $1"
   elif test "$MYCONFIG" != ""; then
     cmd make -j$MAKEJ || die "Unable to build $1"
   fi
@@ -952,28 +1017,8 @@ function configure_compile_install_subproject {
   feedback_status Installing $1
   export LD_LIBRARY_PATH=`lookup LD_LIBRARY_PATH_ $1`
   log "LD_LIBRARY_PATH=$LD_LIBRARY_PATH"
-  if test "shotcut" = "$1" ; then
-    cmd ninja install
-    cmd install -c COPYING "$FINAL_INSTALL_DIR"
-    cmd install -c packaging/windows/shotcut.nsi "$FINAL_INSTALL_DIR"/..
-    cmd sed -i "s/YY.MM.DD/$SHOTCUT_VERSION/" "$FINAL_INSTALL_DIR"/../shotcut.nsi
-    cmd install -d "$FINAL_INSTALL_DIR"/share/translations
-    cmd install -p -c translations/*.qm "$FINAL_INSTALL_DIR"/share/translations
-  elif test "frei0r" = "$1" -o "aom" = "$1" -o "mlt" = "$1"; then
-    cmd ninja install || die "Unable to install $1"
-  elif test "bigsh0t" = "$1" ; then
-    cmd install -p -c *.dll "$FINAL_INSTALL_DIR"/lib/frei0r-1  || die "Unable to install $1"
-  elif test "dav1d" = "$1"; then
-    cmd meson install -C builddir || die "Unable to install $1"
-  elif test "vmaf" = "$1"; then
-    cmd ninja install -C libvmaf/build || die "Unable to install $1"
-    cmd install -d "$FINAL_INSTALL_DIR"/share/vmaf
-    cmd install -p -c model/*.json "$FINAL_INSTALL_DIR"/share/vmaf || die "Unable to install $1"
-  elif test "glaxnimate" = "$1"; then
-    cmd ninja translations || die "Unable to build translations for $1"
-    cmd ninja install || die "Unable to install $1"
-  elif test "gopro2gpx" = "$1" ; then
-    cmd install -p -c gopro2gpx "$FINAL_INSTALL_DIR" || die "Unable to install $1"
+  if test "$MYINSTALL" != "" ; then
+    cmd $MYINSTALL || die "Unable to install $1"
   elif test "$MYCONFIG" != "" ; then
     cmd make install || die "Unable to install $1"
   fi
@@ -1098,6 +1143,8 @@ function deploy
   cmd mv lib/libCuteLogger.dll .
   if [ "$SDK" = "1" ]; then
     cmd mv bin/*.exe .
+    # OpenCV installs binaries to a weird place
+    cmd cp x64/mingw/bin/*.dll .
   else
     cmd mv bin/ffmpeg.exe .
     cmd mv bin/ffplay.exe .
@@ -1107,15 +1154,17 @@ function deploy
     cmd rm lib/*
     cmd rm -rf lib/cmake lib/pkgconfig lib/gdk-pixbuf-2.0 lib/glib-2.0 lib/gtk-2.0
     cmd rm -rf share/doc share/man share/ffmpeg/examples share/aclocal share/glib-2.0 share/gtk-2.0 share/gtk-doc share/themes share/locale
+    # OpenCV installs binaries to a weird place
+    cmd mv x64/mingw/bin/*.dll .
+    cmd rm -rf x64
+    cmd rm OpenCVConfig*
+    cmd rm setup_vars_opencv4.cmd
   fi
   cmd mv COPYING COPYING.txt
 
   log Copying some libs from Qt
   if [ "$DEBUG_BUILD" != "1" -o "$SDK" = "1" ]; then
-    cmd cp -p "$QTDIR"/bin/Qt5{Core,Gui,Multimedia,Network,OpenGL,Qml,QmlModels,QmlWorkerScript,Quick,QuickControls2,QuickTemplates2,QuickWidgets,Sql,Svg,WebSockets,Widgets,WinExtras,Xml}.dll .
-  fi
-  if [ "$DEBUG_BUILD" = "1" -o "$SDK" = "1" ]; then
-    cmd cp -p "$QTDIR"/bin/Qt5{Core,Gui,Multimedia,Network,OpenGL,Qml,QmlModels,QmlWorkerScript,Quick,QuickControls2,QuickTemplates2,QuickWidgets,Sql,Svg,WebSockets,Widgets,WinExtras,Xml}d.dll .
+    cmd cp -p "$QTDIR"/bin/Qt6{Charts,Core,Core5Compat,Gui,Multimedia,Network,OpenGL,OpenGLWidgets,Qml,QmlModels,QmlWorkerScript,Quick,QuickControls2,QuickControls2Impl,QuickDialogs2,QuickDialogs2QuickImpl,QuickDialogs2Utils,QuickLayouts,QuickTemplates2,QuickWidgets,Sql,Svg,SvgWidgets,UiTools,Widgets,Xml}.dll .
   fi
 
   if [ "ENABLE_SWH_PLUGINS" != "1" ]; then
@@ -1123,9 +1172,6 @@ function deploy
     cmd tar -xJf "$HOME/swh-plugins-win64-0.4.15.tar.xz"
   fi
 
-  log Copying some libs from mlt-prebuilt
-  cmd cp -p /mingw64/bin/.dll
-
   for lib in *.dll; do
     bundle_dlls "$lib"
   done
@@ -1133,114 +1179,55 @@ function deploy
     bundle_dlls "$lib"
   done
   bundle_dlls glaxnimate.exe
-  cmd rm *.bundled
 
   log Copying some DLLs and python libraries for Glaxnimate
-  cmd cp -p /mingw64/bin/libpython3.10.dll python310.dll
+  cmd cp -p /mingw64/bin/libpython3.11.dll python311.dll
   cmd cp -p "$SOURCE_DIR"/glaxnimate/external/Qt-Color-Widgets/libQtColorWidgets.dll .
   cmd mkdir -p share/glaxnimate/glaxnimate/pythonhome/lib/python
-  cmd cp -r /mingw64/lib/python3.10/*.py \
-            /mingw64/lib/python3.10/lib-dynload/* \
-            /mingw64/lib/python3.10/{json,collections,encodings,logging,urllib} \
+  cmd cp -r /mingw64/lib/python3.11/*.py \
+            /mingw64/lib/python3.11/lib-dynload/* \
+            /mingw64/lib/python3.11/{json,collections,encodings,logging,urllib} \
       share/glaxnimate/glaxnimate/pythonhome/lib/python
 
+  log Copying some libs from mlt-prebuilt
   cmd cp -p "$HOME"/bin/*.dll .
-  cmd cp -p "$QTDIR"/bin/{libEGL,libGLESv2,d3dcompiler_47}.dll .
+  cmd cp -p "$QTDIR"/bin/d3dcompiler_47.dll .
 
   log Copying some libs from msys2
   cmd cp -p /mingw64/bin/{libcrypto-1_1-x64,libssl-1_1-x64}.dll .
   if [ "$DEBUG_BUILD" = "1" -o "$SDK" = "1" ]; then
     cmd cp -p "$SOURCE_DIR"/shotcut/drmingw/x64/bin/*.{dll,yes} .
+    cmd cp -p /mingw64/bin/libfftw3*.dll bin/
+    cmd cp -p /mingw64/lib/libfftw3*.a lib/
+    cmd cp -p /mingw64/include/fftw3* include/
+    cmd cp -p /mingw64/lib/pkgconfig/fftw3*.pc lib/pkgconfig/
+#    cmd mkdir -p lib/cmake/fftw3 lib/cmake/fftw3f lib/cmake/fftw3l lib/cmake/fftw3q
+#    cmd cp -p /mingw64/lib/cmake/fftw3/*.cmake lib/cmake/fftw3/
+#    cmd cp -p /mingw64/lib/cmake/fftw3f/*.cmake lib/cmake/fftw3f/
+#    cmd cp -p /mingw64/lib/cmake/fftw3l/*.cmake lib/cmake/fftw3l/
+#    cmd cp -p /mingw64/lib/cmake/fftw3q/*.cmake lib/cmake/fftw3q/
   fi
 
   log Copying some plugins, qml, and translations from Qt
-  cmd mkdir -p lib/qt5/sqldrivers
-  cmd cp -pr "$QTDIR"/plugins/{audio,generic,iconengines,imageformats,mediaservice,platforms,sqldrivers,styles} lib/qt5
-  cmd cp -pr "$QTDIR"/qml lib
-  sed -i "s/onClicked()/onClicked(mouse)/" lib/qml/QtQuick/Controls/Private/EditMenu_base.qml
-  cmd rm lib/qml/QtQuick/Controls/Private/EditMenu_base.qmlc
+  cmd mkdir -p lib/qt6/audio
+  for plugin in audio generic iconengines imageformats multimedia platforms sqldrivers styles tls; do
+    cmd cp -pr "$QTDIR"/plugins/${plugin} lib/qt6
+    for lib in lib/qt6/${plugin}/*.dll; do
+      bundle_dlls "$lib"
+    done
+  done
+  cmd mkdir -p lib/qml
+  cmd cp -pr "$QTDIR"/qml/{Qt,QtCore,QtQml,QtQuick} lib/qml
   cmd cp -pr "$QTDIR"/translations/qt_*.qm share/translations
   cmd cp -pr "$QTDIR"/translations/qtbase_*.qm share/translations
 
   log Removing things not needed
-  cmd rm lib/qt5/sqldrivers/{qsqlodbc,qsqlodbcd,qsqlpsql,qsqlpsqld}.dll
+  cmd rm *.bundled
+  cmd rm lib/qt6/sqldrivers/{qsqlodbc,qsqlpsql}.dll
   # colortap is not used by Shotcut and no plans to include it, but
   # Kaspersky antivirus is flagging it and scaring people.
   cmd rm lib/frei0r-1/colortap.dll
-  if [ "$DEBUG_BUILD" != "1" -a "$SDK" != "1" ]; then
-    cmd rm lib/qt5/audio/qtaudio_windowsd.dll
-    cmd rm lib/qt5/generic/qtuiotouchplugind.dll
-    cmd rm lib/qt5/iconengines/qsvgicond.dll
-    cmd rm lib/qt5/imageformats/qddsd.dll
-    cmd rm lib/qt5/imageformats/qgifd.dll
-    cmd rm lib/qt5/imageformats/qicod.dll
-    cmd rm lib/qt5/imageformats/qicnsd.dll
-    cmd rm lib/qt5/imageformats/qjpegd.dll
-    cmd rm lib/qt5/imageformats/qsvgd.dll
-    cmd rm lib/qt5/imageformats/qtgad.dll
-    cmd rm lib/qt5/imageformats/qtiffd.dll
-    cmd rm lib/qt5/imageformats/qwbmpd.dll
-    cmd rm lib/qt5/imageformats/qwebpd.dll
-    cmd rm lib/qt5/mediaservice/dsengined.dll
-    cmd rm lib/qt5/mediaservice/qtmedia_audioengined.dll
-    cmd rm lib/qt5/platforms/qminimald.dll
-    cmd rm lib/qt5/platforms/qoffscreend.dll
-    cmd rm lib/qt5/platforms/qwindowsd.dll
-    cmd rm lib/qt5/sqldrivers/qsqlited.dll
-
-    cmd rm lib/qml/QtLocation/declarative_locationd.dll
-    cmd rm lib/qml/QtQml/StateMachine/qtqmlstatemachined.dll
-    cmd rm lib/qml/QtQml/Models.2/modelsplugind.dll
-    cmd rm lib/qml/QtCanvas3D/qtcanvas3dd.dll
-    cmd rm lib/qml/QtPositioning/declarative_positioningd.dll
-    cmd rm lib/qml/QtWinExtras/qml_winextrasd.dll
-    cmd rm lib/qml/QtQuick.2/qtquick2plugind.dll
-    cmd rm lib/qml/Qt/labs/settings/qmlsettingsplugind.dll
-    cmd rm lib/qml/Qt/labs/folderlistmodel/qmlfolderlistmodelplugind.dll
-    cmd rm lib/qml/QtWebSockets/declarative_qmlwebsocketsd.dll
-    cmd rm lib/qml/QtBluetooth/declarative_bluetoothd.dll
-    cmd rm lib/qml/QtTest/qmltestplugind.dll
-    cmd rm lib/qml/QtQuick/LocalStorage/qmllocalstorageplugind.dll
-    cmd rm lib/qml/QtQuick/Window.2/windowplugind.dll
-    cmd rm lib/qml/QtQuick/Dialogs/dialogplugind.dll
-    cmd rm lib/qml/QtQuick/Dialogs/Private/dialogsprivateplugind.dll
-    cmd rm lib/qml/QtQuick/PrivateWidgets/widgetsplugind.dll
-    cmd rm lib/qml/QtQuick/Layouts/qquicklayoutsplugind.dll
-    cmd rm lib/qml/QtQuick/Controls/qtquickcontrolsplugind.dll
-    cmd rm lib/qml/QtQuick/Controls/Styles/Flat/qtquickextrasflatplugind.dll
-    cmd rm lib/qml/QtQuick/Particles.2/particlesplugind.dll
-    cmd rm lib/qml/QtQuick/Extras/qtquickextrasplugind.dll
-    cmd rm lib/qml/QtQuick/XmlListModel/qmlxmllistmodelplugind.dll
-    cmd rm lib/qml/QtSensors/declarative_sensorsd.dll
-    cmd rm lib/qml/QtMultimedia/declarative_multimediad.dll
-    cmd rm lib/qml/QtNfc/declarative_nfcd.dll
-    cmd rm lib/qml/QtWebChannel/declarative_webchanneld.dll
-    cmd rm lib/qml/Qt/labs/calendar/qtlabscalendarplugind.dll
-    cmd rm lib/qml/Qt/labs/platform/qtlabsplatformplugind.dll
-    cmd rm lib/qml/Qt/labs/sharedimage/sharedimageplugind.dll
-    cmd rm lib/qml/Qt3D/Animation/quick3danimationplugind.dll
-    cmd rm lib/qml/Qt3D/Core/quick3dcoreplugind.dll
-    cmd rm lib/qml/Qt3D/Extras/quick3dextrasplugind.dll
-    cmd rm lib/qml/Qt3D/Input/quick3dinputplugind.dll
-    cmd rm lib/qml/Qt3D/Logic/quick3dlogicplugind.dll
-    cmd rm lib/qml/Qt3D/Render/quick3drenderplugind.dll
-    cmd rm lib/qml/QtCharts/qtchartsqml2d.dll
-    cmd rm lib/qml/QtDataVisualization/datavisualizationqml2d.dll
-    cmd rm lib/qml/QtGamepad/declarative_gamepadd.dll
-    cmd rm lib/qml/QtGraphicalEffects/private/qtgraphicaleffectsprivated.dll
-    cmd rm lib/qml/QtGraphicalEffects/qtgraphicaleffectsplugind.dll
-    cmd rm lib/qml/QtPurchasing/declarative_purchasingd.dll
-    cmd rm lib/qml/QtQml/RemoteObjects/qtqmlremoteobjectsd.dll
-    cmd rm lib/qml/QtQuick/Controls.2/Material/qtquickcontrols2materialstyleplugind.dll
-    cmd rm lib/qml/QtQuick/Controls.2/qtquickcontrols2plugind.dll
-    cmd rm lib/qml/QtQuick/Controls.2/Universal/qtquickcontrols2universalstyleplugind.dll
-    cmd rm lib/qml/QtQuick/Scene2D/qtquickscene2dplugind.dll
-    cmd rm lib/qml/QtQuick/Scene3D/qtquickscene3dplugind.dll
-    cmd rm lib/qml/QtQuick/Templates.2/qtquicktemplates2plugind.dll
-    cmd rm lib/qml/QtQuick/VirtualKeyboard/Styles/qtvirtualkeyboardstylesplugind.dll
-    cmd rm lib/qml/QtScxml/declarative_scxmld.dll
-  fi
-  printf "[Paths]\nPlugins=lib/qt5\nQml2Imports=lib/qml\n" > qt.conf
+  printf "[Paths]\nPlugins=lib/qt6\nQml2Imports=lib/qml\n" > qt.conf
 
   if [ "$SDK" = "1" ]; then
     # Prepare src for archiving
@@ -1252,7 +1239,7 @@ function deploy
 
     log Creating archive
     cmd cd ..
-    cmd zip -gr shotcut-sdk.zip Shotcut
+    cmd tar -cJf shotcut-sdk.txz Shotcut
     fi
   popd
 }
