@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2022 Meltytech, LLC
+ * Copyright (c) 2013-2024 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,13 +25,16 @@
 #include <QUndoCommand>
 #include <QString>
 #include <QObject>
+#include <QUuid>
 #include <MltTransition.h>
 #include <MltProducer.h>
 
+#include <vector>
+
 namespace Timeline {
 
 enum {
-    UndoIdTrimClipIn,
+    UndoIdTrimClipIn = 100,
     UndoIdTrimClipOut,
     UndoIdFadeIn,
     UndoIdFadeOut,
@@ -43,11 +46,31 @@ enum {
     UndoIdMoveClip
 };
 
+struct ClipPosition {
+    ClipPosition(int track, int clip)
+    {
+        trackIndex = track;
+        clipIndex = clip;
+    }
+
+    bool operator < (const ClipPosition &rhs) const
+    {
+        if (trackIndex == rhs.trackIndex) {
+            return clipIndex < rhs.clipIndex;
+        } else {
+            return trackIndex < rhs.trackIndex;
+        }
+    }
+
+    int trackIndex;
+    int clipIndex;
+};
+
 class AppendCommand : public QUndoCommand
 {
 public:
     AppendCommand(MultitrackModel &model, int trackIndex, const QString &xml, bool skipProxy = false,
-                  QUndoCommand *parent = 0);
+                  bool seek = true, QUndoCommand *parent = 0);
     void redo();
     void undo();
 private:
@@ -56,6 +79,7 @@ private:
     QString m_xml;
     UndoHelper m_undoHelper;
     bool m_skipProxy;
+    bool m_seek;
 };
 
 class InsertCommand : public QUndoCommand
@@ -128,6 +152,31 @@ private:
     QList<Markers::Marker> m_markers;
 };
 
+class GroupCommand : public QUndoCommand
+{
+public:
+    GroupCommand(MultitrackModel &model, QUndoCommand *parent = 0);
+    void addToGroup(int trackIndex, int clipIndex);
+    void redo();
+    void undo();
+private:
+    MultitrackModel &m_model;
+    QList<ClipPosition> m_clips;
+    QMap<ClipPosition, int> m_prevGroups;
+};
+
+class UngroupCommand : public QUndoCommand
+{
+public:
+    UngroupCommand(MultitrackModel &model, QUndoCommand *parent = 0);
+    void removeFromGroup(int trackIndex, int clipIndex);
+    void redo();
+    void undo();
+private:
+    MultitrackModel &m_model;
+    QMap<ClipPosition, int> m_prevGroups;
+};
+
 class NameTrackCommand : public QUndoCommand
 {
 public:
@@ -208,14 +257,11 @@ private:
 class MoveClipCommand : public QUndoCommand
 {
 public:
-    MoveClipCommand(MultitrackModel &model, MarkersModel &markersModel, int trackDelta, bool ripple,
+    MoveClipCommand(TimelineDock &timeline, int trackDelta, int positionDelta, bool ripple,
                     QUndoCommand *parent = 0);
+    void addClip(int trackIndex, int clipIndex);
     void redo();
     void undo();
-    QMultiMap<int, Mlt::Producer> &selection()
-    {
-        return m_selection;
-    }
 protected:
     int id() const
     {
@@ -224,20 +270,39 @@ protected:
     bool mergeWith(const QUndoCommand *other);
 private:
     void redoMarkers();
+
+    TimelineDock &m_timeline;
     MultitrackModel &m_model;
     MarkersModel &m_markersModel;
+
+    struct Info {
+        int trackIndex;
+        int clipIndex;
+        int frame_in;
+        int frame_out;
+        int start;
+        int group;
+        QUuid uuid;
+
+        Info()
+            : trackIndex(-1)
+            , clipIndex(-1)
+            , frame_in(-1)
+            , frame_out(-1)
+            , start(0)
+            , group(-1)
+        {}
+    };
+
     int m_trackDelta;
+    int m_positionDelta;
     bool m_ripple;
     bool m_rippleAllTracks;
     bool m_rippleMarkers;
     UndoHelper m_undoHelper;
-    QMultiMap<int, Mlt::Producer> m_selection; // ordered by position
+    QMultiMap<int, Info> m_clips; // ordered by position
     bool m_redo;
-    int m_start;
-    int m_trackIndex;
-    int m_clipIndex;
-    int m_markerOldStart;
-    int m_markerNewStart;
+    int m_earliestStart;
     QList<Markers::Marker> m_markers;
 };
 
@@ -313,14 +378,16 @@ private:
 class SplitCommand : public QUndoCommand
 {
 public:
-    SplitCommand(MultitrackModel &model, int trackIndex, int clipIndex, int position,
+    SplitCommand(MultitrackModel &model, const std::vector<int> &trackIndex,
+                 const std::vector<int> &clipIndex,
+                 int position,
                  QUndoCommand *parent = 0);
     void redo();
     void undo();
 private:
     MultitrackModel &m_model;
-    int m_trackIndex;
-    int m_clipIndex;
+    std::vector<int> m_trackIndex;
+    std::vector<int> m_clipIndex;
     int m_position;
     UndoHelper m_undoHelper;
 };
@@ -466,7 +533,7 @@ class RemoveTransitionByTrimInCommand : public TrimCommand
 {
 public:
     RemoveTransitionByTrimInCommand(MultitrackModel &model, int trackIndex, int clipIndex, int delta,
-                                    bool redo = true, QUndoCommand *parent = 0);
+                                    QString xml, bool redo = true, QUndoCommand *parent = 0);
     void redo();
     void undo();
 private:
@@ -474,6 +541,7 @@ private:
     int m_trackIndex;
     int m_clipIndex;
     int m_delta;
+    QString m_xml;
     bool m_redo;
 };
 
@@ -481,7 +549,7 @@ class RemoveTransitionByTrimOutCommand : public TrimCommand
 {
 public:
     RemoveTransitionByTrimOutCommand(MultitrackModel &model, int trackIndex, int clipIndex, int delta,
-                                     bool redo = true, QUndoCommand *parent = 0);
+                                     QString xml, bool redo = true, QUndoCommand *parent = 0);
     void redo();
     void undo();
 private:
@@ -489,6 +557,7 @@ private:
     int m_trackIndex;
     int m_clipIndex;
     int m_delta;
+    QString m_xml;
     bool m_redo;
 };
 
@@ -525,6 +594,7 @@ private:
     MultitrackModel &m_model;
     int m_trackIndex;
     bool m_isVideo;
+    QUuid m_uuid;
 };
 
 class InsertTrackCommand : public QUndoCommand
@@ -538,6 +608,7 @@ private:
     MultitrackModel &m_model;
     int m_trackIndex;
     TrackType m_trackType;
+    QUuid m_uuid;
 };
 
 class RemoveTrackCommand : public QUndoCommand
@@ -553,6 +624,7 @@ private:
     QString m_trackName;
     UndoHelper m_undoHelper;
     QScopedPointer<Mlt::Producer> m_filtersProducer;
+    QUuid m_uuid;
 };
 
 class MoveTrackCommand : public QUndoCommand
@@ -620,12 +692,12 @@ private:
 class DetachAudioCommand: public QUndoCommand
 {
 public:
-    DetachAudioCommand(MultitrackModel &model, int trackIndex, int clipIndex, int position,
+    DetachAudioCommand(TimelineDock &timeline, int trackIndex, int clipIndex, int position,
                        const QString &xml, QUndoCommand *parent = 0);
     void redo();
     void undo();
 private:
-    MultitrackModel &m_model;
+    TimelineDock &m_timeline;
     int m_trackIndex;
     int m_clipIndex;
     int m_position;
@@ -633,6 +705,7 @@ private:
     QString m_xml;
     UndoHelper m_undoHelper;
     bool m_trackAdded;
+    QUuid m_uuid;
 };
 
 class ReplaceCommand : public QUndoCommand
