@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2022 Meltytech, LLC
+ * Copyright (c) 2012-2023 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,10 +24,10 @@
 #include <QDomElement>
 #include <QStandardItemModel>
 #include <QSortFilterProxyModel>
+#include <QStringList>
 #include <MltProperties.h>
 
 class QTreeWidgetItem;
-class QStringList;
 namespace Ui {
 class EncodeDock;
 }
@@ -119,7 +119,7 @@ private slots:
 
     void on_fpsSpinner_editingFinished();
 
-    void on_fpsComboBox_activated(const QString &arg1);
+    void on_fpsComboBox_activated(int arg1);
 
     void on_videoQualitySpinner_valueChanged(int vq);
 
@@ -127,7 +127,7 @@ private slots:
 
     void on_parallelCheckbox_clicked(bool checked);
 
-    void on_resolutionComboBox_activated(const QString &arg1);
+    void on_resolutionComboBox_activated(int arg1);
 
 private:
     enum {
@@ -150,6 +150,9 @@ private:
     QStringList m_outputFilenames;
     bool m_isDefaultSettings;
     double m_fps;
+    QStringList m_intraOnlyCodecs;
+    QStringList m_losslessVideoCodecs;
+    QStringList m_losslessAudioCodecs;
 
     void loadPresets();
     Mlt::Properties *collectProperties(int realtime, bool includeProfile = false);
@@ -163,9 +166,10 @@ private:
     void resetOptions();
     Mlt::Producer *fromProducer() const;
     static void filterX265Params(QStringList &other);
-    void onVideoCodecComboChanged(int index, bool ignorePreset = false);
+    void onVideoCodecComboChanged(int index, bool ignorePreset = false, bool resetBframes = true);
     bool checkForMissingFiles();
     QString &defaultFormatExtension();
+    void initSpecialCodecLists();
 };
 
 #endif // ENCODEDOCK_H
