@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2022 Meltytech, LLC
+ * Copyright (c) 2013-2023 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,8 @@ class TimelineDock : public QDockWidget
     Q_PROPERTY(QVariantList selection READ selectionForJS WRITE setSelectionFromJS NOTIFY
                selectionChanged)
     Q_PROPERTY(bool isRecording READ isRecording NOTIFY isRecordingChanged)
+    Q_PROPERTY(int loopStart READ loopStart NOTIFY loopChanged)
+    Q_PROPERTY(int loopEnd READ loopEnd NOTIFY loopChanged)
 
 public:
     explicit TimelineDock(QWidget *parent = 0);
@@ -79,8 +81,10 @@ public:
     QVariantList selectionForJS() const;
     const QList<QPoint> selection() const;
     const QVector<QUuid> selectionUuids();
+    const QList<QPoint> uuidsToSelection(QVector<QUuid> uuids) const;
     void saveAndClearSelection();
     Q_INVOKABLE void restoreSelection();
+    Q_INVOKABLE QVariantList getGroupForClip(int trackIndex, int clipIndex);
     void selectClipUnderPlayhead();
     int centerOfClip(int trackIndex, int clipIndex);
     bool isTrackLocked(int trackIndex) const;
@@ -106,12 +110,23 @@ public:
     {
         return m_isRecording;
     }
+    int addTrackIfNeeded(TrackType trackType);
+    void getSelectionRange(int *start, int *end);
+    int loopStart() const
+    {
+        return m_loopStart;
+    }
+    int loopEnd() const
+    {
+        return m_loopEnd;
+    }
 
 signals:
     void currentTrackChanged();
     void selectionChanged();
     void seeked(int position);
     void positionChanged();
+    void loopChanged();
     void clipOpened(Mlt::Producer *producer);
     void dragging(const QPointF &pos, int duration);
     void dropped();
@@ -139,8 +154,8 @@ signals:
     void updateThumbnails(int trackIndex, int clipIndex);
 
 public slots:
-    void addAudioTrack();
-    void addVideoTrack();
+    int addAudioTrack();
+    int addVideoTrack();
     void alignSelectedClips();
     void onShowFrame(const SharedFrame &frame);
     void onSeeked(int position);
@@ -166,7 +181,6 @@ public slots:
     void insert(int trackIndex, int position = -1, const QString &xml = QString(), bool seek = true);
     void overwrite(int trackIndex, int position = -1, const QString &xml = QString(), bool seek = true);
     void appendFromPlaylist(Mlt::Playlist *playlist, bool skipProxy);
-    void splitClip(int trackIndex = -1, int clipIndex = -1);
     void fadeIn(int trackIndex, int clipIndex = -1, int duration = -1);
     void fadeOut(int trackIndex, int clipIndex = -1, int duration = -1);
     void seekPreviousEdit();
@@ -191,7 +205,6 @@ public slots:
     void detachAudio(int trackIndex, int clipIndex);
     void selectAll();
     void selectAllOnCurrentTrack();
-    bool blockSelection(bool block);
     void onProducerModified();
     void replace(int trackIndex, int clipIndex, const QString &xml = QString());
     void createOrEditMarker();
@@ -204,6 +217,9 @@ public slots:
     void onFilterModelChanged();
     void trimClipIn(bool ripple = false);
     void trimClipOut(bool ripple = false);
+    void initLoad();
+    void handleDrop(int trackIndex, int position, QString xml);
+    void onLoopChanged(int start, int end);
 
 protected:
     void dragEnterEvent(QDragEnterEvent *event);
@@ -226,12 +242,13 @@ private:
     {
         return m_model.tractor() && !m_model.trackList().empty();
     }
+    void reportSelectionChange();
 
     QQuickWidget m_quickView;
     MultitrackModel m_model;
     MarkersModel m_markersModel;
     int m_position;
-    QScopedPointer<Timeline::UpdateCommand> m_updateCommand;
+    std::unique_ptr<Timeline::UpdateCommand> m_updateCommand;
     bool m_ignoreNextPositionChange;
     struct Selection {
         QList<QPoint> selectedClips; // x is the clip index, y is the track index
@@ -242,13 +259,14 @@ private:
     int m_savedSelectedTrack;
     bool m_savedIsMultitrackSelected;
     QVector<QUuid> m_savedSelectionUuids;
-    QScopedPointer<Timeline::TrimCommand> m_trimCommand;
-    QScopedPointer<UndoHelper> m_undoHelper;
+    std::unique_ptr<Timeline::TrimCommand> m_trimCommand;
+    std::unique_ptr<UndoHelper> m_undoHelper;
     int m_trimDelta;
     int m_transitionDelta;
-    bool m_blockSetSelection;
+    int m_selectionSilenceCounter;
+    bool m_selectionChanged;
     bool m_isRecording {false};
-    QScopedPointer<AbstractJob> m_recordJob;
+    std::unique_ptr<AbstractJob> m_recordJob;
     QTimer m_recordingTimer;
     QDateTime m_recordingTime;
     int m_recordingTrackIndex;
@@ -256,9 +274,11 @@ private:
     int m_currentTrack {0};
     QMenu *m_mainMenu;
     QMenu *m_clipMenu;
+    int m_loopStart;
+    int m_loopEnd;
 
 private slots:
-    void load(bool force = false);
+    void load(bool force);
     void onTopLevelChanged(bool floating);
     void onTransitionAdded(int trackIndex, int clipIndex, int position, bool ripple);
     void selectClip(int trackIndex, int clipIndex);
@@ -270,20 +290,23 @@ private slots:
     void onWarnTrackLocked();
     void onTimelineRightClicked();
     void onClipRightClicked();
+    void onNoMoreEmptyTracks(bool isAudio);
+
+    friend class TimelineSelectionSilencer;
 };
 
-class TimelineSelectionBlocker
+class TimelineSelectionSilencer
 {
 public:
-    TimelineSelectionBlocker(TimelineDock &timeline)
+    TimelineSelectionSilencer(TimelineDock &timeline)
         : m_timelineDock(timeline)
     {
-        m_timelineDock.blockSelection(true);
+        m_timelineDock.m_selectionSilenceCounter++;
     }
-    ~TimelineSelectionBlocker()
+    ~TimelineSelectionSilencer()
     {
-        QCoreApplication::processEvents();
-        m_timelineDock.blockSelection(false);
+        m_timelineDock.m_selectionSilenceCounter--;
+        m_timelineDock.reportSelectionChange();
     }
 
 private:
