@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2022 Meltytech, LLC
+ * Copyright (c) 2011-2024 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include <framework/mlt_log.h>
 #include <QFile>
 #include <QQuickStyle>
+#include <QQuickWindow>
 
 #ifdef Q_OS_MAC
 #include "macos.h"
@@ -46,11 +47,6 @@ extern "C"
 #endif
 
 static const int kMaxCacheCount = 5000;
-#ifdef Q_OS_WIN
-static const char *kDefaultScaleRoundPolicy = "PassThrough";
-#else
-static const char *kDefaultScaleRoundPolicy = "Round";
-#endif
 
 static void mlt_log_handler(void *service, int mlt_level, const char *format, va_list args)
 {
@@ -86,7 +82,7 @@ static void mlt_log_handler(void *service, int mlt_level, const char *format, va
         if (!resource || resource[0] != '<' || resource[strlen(resource) - 1] != '>')
             mlt_type = mlt_properties_get(properties, "mlt_type" );
         if (service_name)
-            message = QString("[%1 %2] ").arg(mlt_type).arg(service_name);
+            message = QString("[%1 %2] ").arg(mlt_type, service_name);
         else
             message = QString::asprintf("[%s %p] ", mlt_type, service);
         if (resource)
@@ -104,7 +100,7 @@ static void mlt_log_handler(void *service, int mlt_level, const char *format, va
 class Application : public QApplication
 {
 public:
-    MainWindow *mainWindow;
+    MainWindow *mainWindow {nullptr};
     QTranslator qtTranslator;
     QTranslator qtBaseTranslator;
     QTranslator shotcutTranslator;
@@ -116,18 +112,21 @@ public:
         : QApplication(argc, argv)
     {
         auto appPath = applicationDirPath();
+        QDir dir(appPath);
+
 #ifdef Q_OS_WIN
 #include <winbase.h>
         SetDllDirectoryA(appPath.toLocal8Bit());
-#endif
-        QDir dir(appPath);
-#ifdef Q_OS_MAC
+        CreateMutexA(NULL, FALSE, "Meltytech Shotcut Running Mutex");
+#else
         dir.cdUp();
+#endif
+#ifdef Q_OS_MAC
         dir.cd("PlugIns");
         dir.cd("qt");
 #else
         dir.cd("lib");
-        dir.cd("qt5");
+        dir.cd("qt6");
 #endif
         addLibraryPath(dir.absolutePath());
         setOrganizationName("Meltytech");
@@ -139,12 +138,6 @@ public:
 #endif
         setApplicationName("Shotcut");
         setApplicationVersion(SHOTCUT_VERSION);
-        setAttribute(Qt::AA_UseHighDpiPixmaps);
-        setAttribute(Qt::AA_DontCreateNativeWidgetSiblings);
-        setAttribute(Qt::AA_DisableWindowContextHelpButton);
-#if defined(Q_OS_MAC)
-        setAttribute(Qt::AA_DontShowIconsInMenus);
-#endif
 
         // Process command line options.
         QCommandLineParser parser;
@@ -158,6 +151,9 @@ public:
         QCommandLineOption noupgradeOption("noupgrade",
                                            QCoreApplication::translate("main", "Hide upgrade prompt and menu item."));
         parser.addOption(noupgradeOption);
+        QCommandLineOption glaxnimateOption("glaxnimate",
+                                            QCoreApplication::translate("main", "Run Glaxnimate instead of Shotcut."));
+        parser.addOption(glaxnimateOption);
         QCommandLineOption gpuOption("gpu",
                                      QCoreApplication::translate("main", "Use GPU processing."));
         parser.addOption(gpuOption);
@@ -176,16 +172,37 @@ public:
                                          QCoreApplication::translate("main", "A semicolon-separated list of scale factors for each screen"),
                                          QCoreApplication::translate("main", "list"));
         parser.addOption(scaleOption);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
         QCommandLineOption scalePolicyOption("QT_SCALE_FACTOR_ROUNDING_POLICY",
                                              QCoreApplication::translate("main", "How to handle a fractional display scale: %1")
                                              .arg("Round, Ceil, Floor, RoundPreferFloor, PassThrough"),
-                                             QCoreApplication::translate("main", "string"), kDefaultScaleRoundPolicy);
+                                             QCoreApplication::translate("main", "string"), "PassThrough");
         parser.addOption(scalePolicyOption);
+#if defined(Q_OS_WIN)
+        QCommandLineOption sdlAudioDriverOption("SDL_AUDIODRIVER",
+                                                QCoreApplication::translate("main", "Which operating system audio API to use: %1")
+                                                .arg("directsound, wasapi, winmm"),
+                                                QCoreApplication::translate("main", "string"), "wasapi");
+        parser.addOption(sdlAudioDriverOption);
+#elif defined(Q_OS_LINUX)
+        QCommandLineOption sdlAudioDriverOption("SDL_AUDIODRIVER",
+                                                QCoreApplication::translate("main", "Which operating system audio API to use: %1")
+                                                .arg("alsa, arts, dsp, esd, jack, pipewire, pulseaudio"),
+                                                QCoreApplication::translate("main", "string"), "pulseaudio");
+        parser.addOption(sdlAudioDriverOption);
 #endif
         parser.addPositionalArgument("[FILE]...",
                                      QCoreApplication::translate("main", "Zero or more files or folders to open"));
         parser.process(arguments());
+        if (parser.isSet(glaxnimateOption)) {
+            QStringList args = arguments();
+            if (!args.isEmpty())
+                args.removeFirst();
+            args.removeAll("--glaxnimate");
+            QProcess child;
+            if (child.startDetached(Settings.glaxnimatePath(), args))
+                ::exit(EXIT_SUCCESS);
+        }
+
 #ifdef Q_OS_WIN
         isFullScreen = false;
 #else
@@ -205,6 +222,7 @@ public:
         // Startup logging.
         dir.setPath(Settings.appDataLocation());
         if (!dir.exists()) dir.mkpath(dir.path());
+        QFile::copy(dir.filePath("shotcut-log.txt"), dir.filePath("shotcut-log.bak"));
         const QString logFileName = dir.filePath("shotcut-log.txt");
         QFile::remove(logFileName);
         FileAppender *fileAppender = new FileAppender(logFileName);
@@ -223,39 +241,8 @@ public:
         mlt_log_set_callback(mlt_log_handler);
         cuteLogger->logToGlobalInstance("qml", true);
 
-        // Log some basic info.
-        LOG_INFO() << "Starting Shotcut version" << SHOTCUT_VERSION;
-#if defined (Q_OS_WIN)
-        LOG_INFO() << "Windows version" << QSysInfo::windowsVersion();
-#elif defined(Q_OS_MAC)
-        LOG_INFO() << "macOS version" << QSysInfo::macVersion();
-#else
-        LOG_INFO() << "Linux version";
-#endif
-        LOG_INFO() << "number of logical cores =" << QThread::idealThreadCount();
-        LOG_INFO() << "locale =" << QLocale();
-        LOG_INFO() << "install dir =" << appPath;
-        Settings.log();
-
 #if defined(Q_OS_WIN)
         dir.setPath(appPath);
-        if (!Settings.playerGPU() && Settings.drawMethod() == Qt::AA_UseSoftwareOpenGL) {
-            if (QFile::exists(dir.filePath("opengl32sw.dll"))) {
-                if (!QFile::copy(dir.filePath("opengl32sw.dll"), dir.filePath("opengl32.dll"))) {
-                    LOG_WARNING() << "Failed to copy opengl32sw.dll as opengl32.dll";
-                }
-            }
-        } else if (QFile::exists(dir.filePath("opengl32.dll"))) {
-            if (!QFile::remove(dir.filePath("opengl32.dll"))) {
-                LOG_ERROR() << "Failed to remove opengl32.dll";
-            }
-        }
-        if (Settings.playerGPU()) {
-            QCoreApplication::setAttribute(Qt::AA_UseDesktopOpenGL);
-        } else if (Settings.drawMethod() >= Qt::AA_UseDesktopOpenGL &&
-                   Settings.drawMethod() <= Qt::AA_UseSoftwareOpenGL) {
-            QCoreApplication::setAttribute(Qt::ApplicationAttribute(Settings.drawMethod()));
-        }
 #elif !defined(Q_OS_MAC)
         if (Settings.drawMethod() == Qt::AA_UseSoftwareOpenGL && !Settings.playerGPU()) {
             ::qputenv("LIBGL_ALWAYS_SOFTWARE", "1");
@@ -282,12 +269,12 @@ public:
             locale = "pt";
         else if (locale.startsWith("en_"))
             locale = "en";
-        if (qtTranslator.load("qt_" + locale, QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
+        if (qtTranslator.load("qt_" + locale, QLibraryInfo::path(QLibraryInfo::TranslationsPath)))
             installTranslator(&qtTranslator);
         else if (qtTranslator.load("qt_" + locale, dir.absolutePath()))
             installTranslator(&qtTranslator);
         if (qtBaseTranslator.load("qtbase_" + locale,
-                                  QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
+                                  QLibraryInfo::path(QLibraryInfo::TranslationsPath)))
             installTranslator(&qtBaseTranslator);
         else if (qtBaseTranslator.load("qtbase_" + locale, dir.absolutePath()))
             installTranslator(&qtBaseTranslator);
@@ -320,8 +307,6 @@ int main(int argc, char **argv)
 #ifndef QT_DEBUG
     ::qputenv("QT_LOGGING_RULES", "*.warning=false");
 #endif
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 6, 0))
-    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
     for (int i = 1; i + 1 < argc; i++) {
         if (!::qstrcmp("--QT_SCALE_FACTOR", argv[i]) || !::qstrcmp("--QT_SCREEN_SCALE_FACTORS", argv[i])) {
             QByteArray value(argv[i + 1]);
@@ -330,26 +315,23 @@ int main(int argc, char **argv)
             break;
         }
     }
-#endif
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
-    QByteArray value(kDefaultScaleRoundPolicy);
+    if (!::qEnvironmentVariableIsSet("QT_SCALE_FACTOR_ROUNDING_POLICY")) {
+        for (int i = 1; i + 1 < argc; i++) {
+            if (!::qstrcmp("--QT_SCALE_FACTOR_ROUNDING_POLICY", argv[i])) {
+                ::qputenv("QT_SCALE_FACTOR_ROUNDING_POLICY", argv[i + 1]);
+                break;
+            }
+        }
+    }
+#if defined(Q_OS_WIN) || defined(Q_OS_LINUX)
     for (int i = 1; i + 1 < argc; i++) {
-        if (!::qstrcmp("--QT_SCALE_FACTOR_ROUNDING_POLICY", argv[i])) {
-            value = argv[i + 1];
+        if (!::qstrcmp("--SDL_AUDIODRIVER", argv[i])) {
+            ::qputenv("SDL_AUDIODRIVER", argv[i + 1]);
             break;
         }
     }
-    if (!::qEnvironmentVariableIsSet("QT_SCALE_FACTOR_ROUNDING_POLICY")) {
-        ::qputenv("QT_SCALE_FACTOR_ROUNDING_POLICY", value);
-    }
 #endif
 #ifdef Q_OS_MAC
-#if (QT_VERSION < QT_VERSION_CHECK(5, 13, 0))
-    // Fix launch on Big Sur macOS 11.0
-    // We can probably remove this when upgrade to Qt 5.15 and update build environment.
-    // see https://bugreports.qt.io/browse/QTBUG-87014
-    ::qputenv("QT_MAC_WANTS_LAYER", "1");
-#endif
     // Launcher and Spotlight on macOS are not setting this environment
     // variable needed by setlocale() as used by MLT.
     if (QProcessEnvironment::systemEnvironment().value(MLT_LC_NAME).isEmpty()) {
@@ -368,70 +350,118 @@ int main(int argc, char **argv)
     removeMacosTabBar();
 #endif
 
+#if defined(Q_OS_WIN)
+    // Windows can use Direct3D or OpenGL
+#elif defined(Q_OS_MAC)
+    QQuickWindow::setGraphicsApi(QSGRendererInterface::Metal);
+    QCoreApplication::setAttribute(Qt::AA_DontShowIconsInMenus);
+#else
+    QQuickWindow::setGraphicsApi(QSGRendererInterface::OpenGL);
+    QCoreApplication::setAttribute(Qt::AA_UseDesktopOpenGL);
+    QCoreApplication::setAttribute(Qt::AA_DontCreateNativeWidgetSiblings);
+#endif
+
     Application a(argc, argv);
-    QSplashScreen splash(QPixmap(":/icons/shotcut-logo-320x320.png"));
-
-    // Expire old items from the qmlcache
-    splash.showMessage(QCoreApplication::translate("main", "Expiring cache..."),
-                       Qt::AlignRight | Qt::AlignVCenter);
-    splash.show();
-    a.processEvents();
-    auto dir = QDir(QStandardPaths::standardLocations(QStandardPaths::CacheLocation).constFirst());
-    if (dir.exists() && dir.cd("qmlcache")) {
-        auto ls = dir.entryList(QDir::Files | QDir::Readable | QDir::NoDotAndDotDot, QDir::Time);
-        if (qMax(0, ls.size() - kMaxCacheCount) > 0) {
-            LOG_INFO() << "removing" << qMax(0, ls.size() - kMaxCacheCount) << "from" << dir.path();
-        }
-        for (int i = kMaxCacheCount; i < ls.size(); i++) {
-            QString filePath = dir.filePath(ls[i]);
-            if (!QFile::remove(filePath)) {
-                LOG_WARNING() << "failed to delete" << filePath;
-            }
-            if (i % 1000 == 0) {
-                a.processEvents();
-            }
-        }
-    }
-
-    splash.showMessage(QCoreApplication::translate("main", "Loading plugins..."),
-                       Qt::AlignRight | Qt::AlignVCenter);
-    a.processEvents();
-
-    QQuickStyle::setStyle("Fusion");
-    a.setProperty("system-style", a.style()->objectName());
-    MainWindow::changeTheme(Settings.theme());
-
-    a.mainWindow = &MAIN;
-    if (!a.appDirArg.isEmpty())
-        a.mainWindow->hideSetDataDirectory();
+    int result = EXIT_SUCCESS;
 #ifdef Q_OS_WIN
-    a.mainWindow->setProperty("windowOpacity", 0.0);
+    if (::qEnvironmentVariableIsSet("QSG_RHI_BACKEND")) {
 #endif
-    a.mainWindow->show();
-    a.processEvents();
-    a.mainWindow->setFullScreen(a.isFullScreen);
-    splash.finish(a.mainWindow);
+        QSplashScreen splash(QPixmap(":/icons/shotcut-logo-320x320.png"));
 
-    if (!a.resourceArg.isEmpty())
-        a.mainWindow->openMultiple(a.resourceArg);
-    else
-        a.mainWindow->open(a.mainWindow->untitledFileName());
+        // Log some basic info.
+        LOG_INFO() << "Starting Shotcut version" << SHOTCUT_VERSION;
+#if defined(Q_OS_WIN)
+        LOG_INFO() << "Windows version" << QSysInfo::productVersion();
+#elif defined(Q_OS_MAC)
+        LOG_INFO() << "macOS version" << QSysInfo::productVersion();
+#else
+        LOG_INFO() << "Linux version" << QSysInfo::productVersion();;
+#endif
+        LOG_INFO() << "number of logical cores =" << QThread::idealThreadCount();
+        LOG_INFO() << "locale =" << QLocale();
+        LOG_INFO() << "install dir =" << a.applicationDirPath();
+        Settings.log();
 
-    int result = a.exec();
+        // Expire old items from the qmlcache
+        splash.showMessage(QCoreApplication::translate("main", "Expiring cache..."),
+                           Qt::AlignRight | Qt::AlignVCenter);
+        splash.show();
+        a.processEvents();
+        auto dir = QDir(QStandardPaths::standardLocations(QStandardPaths::CacheLocation).constFirst());
+        if (dir.exists() && dir.cd("qmlcache")) {
+            auto ls = dir.entryList(QDir::Files | QDir::Readable | QDir::NoDotAndDotDot, QDir::Time);
+            if (qMax(0, ls.size() - kMaxCacheCount) > 0) {
+                LOG_INFO() << "removing" << qMax(0, ls.size() - kMaxCacheCount) << "from" << dir.path();
+            }
+            for (int i = kMaxCacheCount; i < ls.size(); i++) {
+                QString filePath = dir.filePath(ls[i]);
+                if (!QFile::remove(filePath)) {
+                    LOG_WARNING() << "failed to delete" << filePath;
+                }
+                if (i % 1000 == 0) {
+                    a.processEvents();
+                }
+            }
+        }
 
-    if (EXIT_RESTART == result || EXIT_RESET == result) {
-        LOG_DEBUG() << "restarting app";
+        splash.showMessage(QCoreApplication::translate("main", "Loading plugins..."),
+                           Qt::AlignRight | Qt::AlignVCenter);
+        a.processEvents();
+
+        a.setProperty("system-style", a.style()->objectName());
+        MainWindow::changeTheme(Settings.theme());
+        QQuickStyle::setStyle("Fusion");
+
+        a.mainWindow = &MAIN;
+        if (!a.appDirArg.isEmpty())
+            a.mainWindow->hideSetDataDirectory();
+#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
+        a.mainWindow->setProperty("windowOpacity", 0.0);
+#endif
+        a.mainWindow->show();
+        a.processEvents();
+        a.mainWindow->setFullScreen(a.isFullScreen);
+        splash.finish(a.mainWindow);
+
+        if (!a.resourceArg.isEmpty())
+            a.mainWindow->openMultiple(a.resourceArg);
+        else
+            a.mainWindow->open(a.mainWindow->untitledFileName());
+
+        result = a.exec();
+
+        if (EXIT_RESTART == result || EXIT_RESET == result) {
+            LOG_DEBUG() << "restarting app";
 #if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
-        ::qputenv("LIBGL_ALWAYS_SOFTWARE",
-                  Settings.drawMethod() == Qt::AA_UseSoftwareOpenGL && !Settings.playerGPU()
-                  ? "1" : "0");
+            ::qputenv("LIBGL_ALWAYS_SOFTWARE",
+                      Settings.drawMethod() == Qt::AA_UseSoftwareOpenGL && !Settings.playerGPU()
+                      ? "1" : "0");
 #endif
-        QProcess *restart = new QProcess;
+            QProcess *restart = new QProcess;
+            QStringList args = a.arguments();
+            if (!args.isEmpty())
+                args.removeFirst();
+            restart->start(a.applicationFilePath(), args, QIODevice::NotOpen);
+            result = EXIT_SUCCESS;
+        }
+
+#ifdef Q_OS_WIN
+    } else { // if (::qEnvironmentVariableIsSet("QSG_RHI_BACKEND"))
+        // Run as a parent process to check if the child crashes on startup
+        QProcess *child = new QProcess;
         QStringList args = a.arguments();
         if (!args.isEmpty())
             args.removeFirst();
-        restart->start(a.applicationFilePath(), args, QIODevice::NotOpen);
-        result = EXIT_SUCCESS;
+        ::qputenv("QSG_RHI_BACKEND", "d3d11");
+        child->start(a.applicationFilePath(), args, QIODevice::NotOpen);
+        child->waitForFinished();
+        if (QProcess::CrashExit == child->exitStatus() || child->exitCode()) {
+            LOG_WARNING() << "child process failed, restarting in OpenGL mode";
+            ::qputenv("QSG_RHI_BACKEND", "opengl");
+            child->start(a.applicationFilePath(), args, QIODevice::NotOpen);
+        }
     }
+#endif
+
     return result;
 }
