@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2021 Meltytech, LLC
+ * Copyright (c) 2011-2024 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -126,6 +126,9 @@ public:
     }
     int mltIndexForTrack(int trackIndex) const;
     int bottomVideoTrackIndex() const;
+    void cropSource(const QRectF &rect);
+    void getMarkerRange(int position, int *start, int *end);
+    void getSelectionRange(int *start, int *end);
 
 signals:
     void audioChannelsChanged();
@@ -155,9 +158,9 @@ private:
     void setupLayoutSwitcher();
     void centerLayoutInRemainingToolbarSpace();
     void setupAndConnectDocks();
+    void setupMenuFile();
     void setupMenuView();
     void connectVideoWidgetSignals();
-    void setupAndConnectLeapNetworkListener();
     void setupSettingsMenu();
     void setupOpenOtherMenu();
     void setupActions();
@@ -181,6 +184,8 @@ private:
     void resetDockCorners();
     void showIncompatibleProjectMessage(const QString &shotcutVersion);
     void restartAfterChangeTheme();
+    void backup();
+    void backupPeriodically();
 
     Ui::MainWindow *ui;
     Player *m_player;
@@ -205,6 +210,7 @@ private:
     QMenu *m_customProfileMenu;
     QMenu *m_keyerMenu;
     QStringList m_multipleFiles;
+    bool m_multipleFilesLoading;
     bool m_isPlaylistLoaded;
     QActionGroup *m_languagesGroup;
     QSharedPointer<AutoSaveFile> m_autosaveFile;
@@ -219,15 +225,13 @@ private:
     QDateTime m_sourceUpdatedAt;
     MarkersDock *m_markersDock;
     NotesDock *m_notesDock;
-
-#ifdef WITH_LIBLEAP
-    LeapListener m_leapListener;
-#endif
+    std::unique_ptr<QWidget> m_producerWidget;
 
 public slots:
     bool isCompatibleWithGpuMode(MltXmlChecker &checker);
     bool isXmlRepaired(MltXmlChecker &checker, QString &fileName);
-    bool open(QString url, const Mlt::Properties * = nullptr, bool play = true);
+    bool open(QString url, const Mlt::Properties * = nullptr, bool play = true,
+              bool skipConvert = false);
     void openMultiple(const QStringList &paths);
     void openMultiple(const QList<QUrl> &urls);
     void openVideo();
@@ -294,6 +298,7 @@ private slots:
     void on_actionLinearBlend_triggered(bool checked);
     void on_actionYadifTemporal_triggered(bool checked);
     void on_actionYadifSpatial_triggered(bool checked);
+    void on_actionBwdif_triggered(bool checked);
     void on_actionNearest_triggered(bool checked);
     void on_actionBilinear_triggered(bool checked);
     void on_actionBicubic_triggered(bool checked);
@@ -306,6 +311,7 @@ private slots:
     void onProfileChanged();
     void on_actionAddCustomProfile_triggered();
     void processMultipleFiles();
+    void processSingleFile();
     void onLanguageTriggered(QAction *);
     void on_actionSystemTheme_triggered();
     void on_actionFusionDark_triggered();
@@ -321,8 +327,6 @@ private slots:
     void on_actionUpgrade_triggered();
     void on_actionOpenXML_triggered();
     void onAutosaveTimeout();
-    void on_actionGammaSRGB_triggered(bool checked);
-    void on_actionGammaRec709_triggered(bool checked);
     void onFocusChanged(QWidget *old, QWidget *now) const;
     void onFocusObjectChanged(QObject *obj) const;
     void onFocusWindowChanged(QWindow *window) const;
@@ -332,6 +336,7 @@ private slots:
 #if !defined(Q_OS_MAC)
     void onDrawingMethodTriggered(QAction *);
 #endif
+    void on_actionResources_triggered();
     void on_actionApplicationLog_triggered();
     void on_actionClose_triggered();
     void onPlayerTabIndexChanged(int index);
@@ -340,7 +345,7 @@ private slots:
     void onClipCopied();
     void on_actionExportEDL_triggered();
     void on_actionExportFrame_triggered();
-    void onGLWidgetImageReady();
+    void onVideoWidgetImageReady();
     void on_actionAppDataSet_triggered();
     void on_actionAppDataShow_triggered();
     void on_actionNew_triggered();
@@ -359,6 +364,7 @@ private slots:
     void on_actionLayoutRemove_triggered();
     void on_actionOpenOther2_triggered();
     void onOpenOtherTriggered(QWidget *widget);
+    void onOpenOtherFinished(int result);
     void onOpenOtherTriggered();
     void on_actionClearRecentOnExit_toggled(bool arg1);
     void onSceneGraphInitialized();
@@ -386,6 +392,7 @@ private slots:
     void on_actionExportChapters_triggered();
     void on_actionAudioVideoDevice_triggered();
     void on_actionReset_triggered();
+    void on_actionBackupSave_triggered();
 };
 
 #define MAIN MainWindow::singleton()
