@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2022 Meltytech, LLC
+ * Copyright (c) 2011-2024 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,14 +32,22 @@
 #include <clocale>
 #include <unistd.h>
 
-#include "glwidget.h"
 #include "settings.h"
 #include "shotcut_mlt_properties.h"
 #include "mainwindow.h"
 #include "controllers/filtercontroller.h"
+#include "dialogs/longuitask.h"
 #include "qmltypes/qmlmetadata.h"
 #include "util.h"
 #include "proxymanager.h"
+#if defined(Q_OS_WIN)
+#include "widgets/d3dvideowidget.h"
+#include "widgets/openglvideowidget.h"
+#elif defined(Q_OS_MAC)
+#include "widgets/metalvideowidget.h"
+#else
+#include "widgets/openglvideowidget.h"
+#endif
 
 namespace Mlt {
 
@@ -54,6 +62,7 @@ Controller::Controller()
     , m_blockRefresh(false)
 {
     LOG_DEBUG() << "begin";
+    ::qputenv("MLT_REPOSITORY_DENY", "libmltqt:libmltglaxnimate");
     m_repo = Mlt::Factory::init();
     resetLocale();
     initFiltersClipboard();
@@ -66,7 +75,16 @@ Controller &Controller::singleton(QObject *parent)
     if (!instance) {
         qRegisterMetaType<Mlt::Frame>("Mlt::Frame");
         qRegisterMetaType<SharedFrame>("SharedFrame");
-        instance = new GLWidget(parent);
+#if defined(Q_OS_WIN)
+        if (QSGRendererInterface::Direct3D11 == QQuickWindow::graphicsApi())
+            instance = new D3DVideoWidget(parent);
+        else
+            instance = new OpenGLVideoWidget(parent);
+#elif defined(Q_OS_MAC)
+        instance = new MetalVideoWidget(parent);
+#else
+        instance = new OpenGLVideoWidget(parent);
+#endif
     }
     return *instance;
 }
@@ -100,12 +118,7 @@ int Controller::setProducer(Mlt::Producer *producer, bool)
     return error;
 }
 
-static bool isFpsDifferent(double a, double b)
-{
-    return qAbs(a - b) > 0.001;
-}
-
-int Controller::open(const QString &url, const QString &urlToSave)
+int Controller::open(const QString &url, const QString &urlToSave, bool skipConvert)
 {
     int error = checkFile(url);
     if (error) {
@@ -113,6 +126,8 @@ int Controller::open(const QString &url, const QString &urlToSave)
     }
 
     Mlt::Producer *newProducer = nullptr;
+    LongUiTask longTask(QObject::tr("Open Project"));
+    longTask.setMinimumDuration(500);
 
     close();
 
@@ -121,12 +136,19 @@ int Controller::open(const QString &url, const QString &urlToSave)
         // MLT xml producer does URL decoding; so if the URL contains % it must be encoded.
         myUrl = QUrl::toPercentEncoding(url).constData();
     }
-    if (Settings.playerGPU() && !profile().is_explicit())
+    QString openMessage = QObject::tr("Opening %1").arg(url.toUtf8().constData());
+    if (Settings.playerGPU() && !profile().is_explicit()) {
         // Prevent loading normalizing filters, which might be Movit ones that
         // may not have a proper OpenGL context when requesting a sample frame.
-        newProducer = new Mlt::Producer(profile(), "abnormal", myUrl.toUtf8().constData());
-    else
-        newProducer = new Mlt::Producer(profile(), myUrl.toUtf8().constData());
+        newProducer = longTask.runAsync<Mlt::Producer *>(openMessage, [ = ]() {
+            return new Mlt::Producer(profile(), "abnormal", myUrl.toUtf8().constData());
+        });
+    } else {
+        newProducer = longTask.runAsync<Mlt::Producer *>(openMessage, [ = ]() {
+            return new Mlt::Producer(profile(), myUrl.toUtf8().constData());
+        });
+    }
+    longTask.reportProgress(QObject::tr("Loading"), 0, 0);
     if (newProducer && newProducer->is_valid()) {
         double fps = profile().fps();
         if (!profile().is_explicit()) {
@@ -150,7 +172,8 @@ int Controller::open(const QString &url, const QString &urlToSave)
                 setProjectFolder(QString());
             }
         }
-        if (isFpsDifferent(profile().fps(), fps) || (Settings.playerGPU() && !profile().is_explicit())) {
+        if (Util::isFpsDifferent(profile().fps(), fps) || (Settings.playerGPU()
+                                                           && !profile().is_explicit())) {
             // Reload with correct FPS or with Movit normalizing filters attached.
             delete newProducer;
             newProducer = new Mlt::Producer(profile(), myUrl.toUtf8().constData());
@@ -162,6 +185,7 @@ int Controller::open(const QString &url, const QString &urlToSave)
                 m_url = urlToSave;
         }
         Producer *producer = setupNewProducer(newProducer);
+        producer->set(kShotcutSkipConvertProperty, skipConvert);
         delete newProducer;
         newProducer = producer;
     } else {
@@ -177,27 +201,33 @@ bool Controller::openXML(const QString &filename)
 {
     bool error = true;
     close();
-    Producer *producer = new Mlt::Producer(profile(), filename.toUtf8().constData());
-    if (producer->is_valid()) {
+    if (Settings.playerGPU() && profile().is_explicit()) {
+        Profile testProfile;
+        Producer producer(testProfile, filename.toUtf8().constData());
+        if (testProfile.width() != profile().width() || testProfile.height() != profile().height()
+                || Util::isFpsDifferent(profile().fps(), testProfile.fps())) {
+            return error;
+        }
+    }
+    Producer producer(profile(), filename.toUtf8().constData());
+    if (producer.is_valid()) {
         double fps = profile().fps();
         if (!profile().is_explicit()) {
-            profile().from_producer(*producer);
+            profile().from_producer(producer);
             profile().set_width(Util::coerceMultiple(profile().width()));
             profile().set_height(Util::coerceMultiple(profile().height()));
         }
         updatePreviewProfile();
         setPreviewScale(Settings.playerPreviewScale());
-        if (isFpsDifferent(profile().fps(), fps)) {
+        if (Util::isFpsDifferent(profile().fps(), fps)) {
             // reopen with the correct fps
-            delete producer;
-            producer = new Mlt::Producer(profile(), filename.toUtf8().constData());
+            producer = Producer(profile(), filename.toUtf8().constData());
         }
-        producer->set(kShotcutVirtualClip, 1);
-        producer->set("resource", filename.toUtf8().constData());
+        producer.set(kShotcutVirtualClip, 1);
+        producer.set("resource", filename.toUtf8().constData());
         setProducer(new Producer(producer));
         error = false;
     }
-    delete producer;
     return error;
 }
 
@@ -491,7 +521,6 @@ bool Controller::saveXML(const QString &filename, Service *service, bool withRel
         if (ignore)
             s.set("ignore_points", 0);
         c.set("time_format", "clock");
-        c.set("no_meta", 1);
         c.set("store", "shotcut");
         c.set("root", root.toUtf8().constData());
         c.set("no_root", 1);
@@ -504,7 +533,7 @@ bool Controller::saveXML(const QString &filename, Service *service, bool withRel
         if (!proxy && ProxyManager::filterXML(xml, root)) { // also verifies
             if (tempFile) {
                 QTextStream stream(tempFile);
-                stream.setCodec("UTF-8");
+                stream.setEncoding(QStringConverter::Utf8);
                 stream << xml;
                 if (tempFile->error() != QFileDevice::NoError) {
                     LOG_ERROR() << "error while writing MLT XML file" << tempFile->fileName() << ":" <<
@@ -519,7 +548,7 @@ bool Controller::saveXML(const QString &filename, Service *service, bool withRel
                     return false;
                 }
                 QTextStream stream(&file);
-                stream.setCodec("UTF-8");
+                stream.setEncoding(QStringConverter::Utf8);
                 stream << xml;
                 if (file.error() != QFileDevice::NoError) {
                     LOG_ERROR() << "error while writing MLT XML file" << filename << ":" << file.errorString();
@@ -636,7 +665,7 @@ bool Controller::isSeekable(Producer *p) const
         } else {
             seekable = producer->get_int("seekable");
             if (!seekable && producer->get("mlt_type")) {
-                // XXX what was this for?
+                // MLT xml producer or tractor
                 seekable = !strcmp(producer->get("mlt_type"), "mlt_producer");
             }
             if (!seekable) {
@@ -644,7 +673,7 @@ bool Controller::isSeekable(Producer *p) const
                 // TODO: Currently, these max out at 15000 frames, which is arbitrary.
                 QString service(producer->get("mlt_service"));
                 seekable = (service == "color") || service.startsWith("frei0r.") || (service == "tone")
-                           || (service == "count") || (service == "noise");
+                           || (service == "count") || (service == "noise") || (service == "consumer");
             }
         }
     }
@@ -960,6 +989,7 @@ Producer *Controller::setupNewProducer(Producer *newProducer) const
             Mlt::Chain *chain = new Mlt::Chain(MLT.profile());
             chain->set_source(*newProducer);
             chain->get_length_time(mlt_time_clock);
+            chain->attach_normalizers();
 
             // Move all non-loader filters to the chain in case this was a clip-only project.
             int i = 0;
@@ -981,7 +1011,7 @@ Producer *Controller::setupNewProducer(Producer *newProducer) const
 
 QUuid Controller::uuid(Mlt::Properties &properties) const
 {
-    return {properties.get(kUuidProperty)};
+    return QUuid(properties.get(kUuidProperty));
 }
 
 void Controller::setUuid(Mlt::Properties &properties, QUuid uid) const
@@ -1053,7 +1083,8 @@ void Controller::copyFilters(Producer &fromProducer, Producer &toProducer, bool
         count = fromChain.link_count();
         for (int i = 0; i < count; i++) {
             QScopedPointer<Mlt::Link> fromLink(fromChain.link(i));
-            if (fromLink && fromLink->is_valid() && fromLink->get("mlt_service")) {
+            if (fromLink && fromLink->is_valid() && fromLink->get("mlt_service")
+                    && !fromLink->get_int("_loader")) {
                 Mlt::Link toLink(fromLink->get("mlt_service"));
                 if (toLink.is_valid()) {
                     toLink.inherit(*fromLink);
@@ -1140,9 +1171,13 @@ void Controller::adjustFilters(Producer &producer, int index)
                     QmlMetadata *meta = MAIN.filterController()->metadataForService(filter.data());
                     if (meta && meta->keyframes()) {
                         foreach (QString name, meta->keyframes()->simpleProperties()) {
-                            if (!filter->get_animation(name.toUtf8().constData()))
+                            if (!filter->get_animation(name.toUtf8().constData())) {
                                 // Cause a string property to be interpreted as animated value.
-                                filter->anim_get_double(name.toUtf8().constData(), 0, filter->get_length());
+                                if (meta->keyframes()->parameter(name)->isColor())
+                                    filter->anim_get_color(name.toUtf8().constData(), 0, filter->get_length());
+                                else
+                                    filter->anim_get_double(name.toUtf8().constData(), 0, filter->get_length());
+                            }
                             Mlt::Animation animation = filter->get_animation(name.toUtf8().constData());
                             if (animation.is_valid()) {
                                 int n = animation.key_count();
@@ -1184,6 +1219,9 @@ static void shiftKeyframes(Mlt::Service *service, QmlMetadata *meta, int delta)
                 if (paramMeta->isRectangle()) {
                     auto value = service->anim_get_rect(qUtf8Printable(name), delta);
                     service->anim_set(qUtf8Printable(name), value, delta, 0, existingType);
+                } else if (paramMeta->isColor()) {
+                    auto value = service->anim_get_color(qUtf8Printable(name), delta);
+                    service->anim_set(qUtf8Printable(name), value, delta, 0, existingType);
                 } else {
                     double value = service->anim_get_double(qUtf8Printable(name), delta);
                     service->anim_set(qUtf8Printable(name), value, delta, 0, existingType);
@@ -1224,7 +1262,8 @@ static void shiftKeyframes(Mlt::Service *service, QmlMetadata *meta, int delta)
             animation.shift_frames(-delta);
             foreach (QString gangName, paramMeta->gangedProperties()) {
                 Mlt::Animation gangAnim = service->get_animation(qUtf8Printable(gangName));
-                gangAnim.shift_frames(-delta);
+                if (gangAnim.is_valid())
+                    gangAnim.shift_frames(-delta);
             }
         }
     }
@@ -1318,9 +1357,13 @@ void Controller::adjustFilter(Mlt::Filter *filter, int in, int out, int inDelta,
             // Update simple keyframes
             if (filter->get_int(kShotcutAnimOutProperty) && meta && meta->keyframes()) {
                 foreach (QString name, meta->keyframes()->simpleProperties()) {
-                    if (!filter->get_animation(name.toUtf8().constData()))
+                    if (!filter->get_animation(name.toUtf8().constData())) {
                         // Cause a string property to be interpreted as animated value.
-                        filter->anim_get_double(name.toUtf8().constData(), 0, filter->get_length());
+                        if (meta->keyframes()->parameter(name)->isColor())
+                            filter->anim_get_color(name.toUtf8().constData(), 0, filter->get_length());
+                        else
+                            filter->anim_get_double(name.toUtf8().constData(), 0, filter->get_length());
+                    }
                     Mlt::Animation animation = filter->get_animation(name.toUtf8().constData());
                     if (animation.is_valid()) {
                         int n = animation.key_count();
@@ -1519,7 +1562,8 @@ int Controller::checkFile(const QString &path)
 {
     int error = 0;
     if (path.endsWith(".json") || path.endsWith(".rawr") || path.endsWith(".lottie")
-            || path.endsWith(".tgs")) {
+            || path.endsWith(".riv") || path.endsWith(".tgs") || path.endsWith(".avd")
+            || path.endsWith(".aep")) {
         QString shotcutPath = qApp->applicationDirPath();
 #if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
         QFileInfo meltPath(shotcutPath, "melt-7");
