@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2021 Meltytech, LLC
+ * Copyright (c) 2013-2023 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <QVariant>
 #include <QRectF>
 #include <QUuid>
+#include <QColor>
 #include <MltService.h>
 #include <MltProducer.h>
 #include <MltAnimation.h>
@@ -32,12 +33,17 @@
 
 class AbstractJob;
 class EncodeJob;
+class QUndoCommand;
+class FilterController;
+namespace Filter {
+class ChangeParameterCommand;
+}
 
 class QmlFilter : public QObject
 {
     Q_OBJECT
-    Q_PROPERTY(bool isNew READ isNew)
-    Q_PROPERTY(QString path READ path)
+    Q_PROPERTY(bool isNew READ isNew CONSTANT)
+    Q_PROPERTY(QString path READ path CONSTANT)
     Q_PROPERTY(QStringList presets READ presets NOTIFY presetsChanged)
     Q_PROPERTY(int in READ in NOTIFY inChanged)
     Q_PROPERTY(int out READ out NOTIFY outChanged)
@@ -76,11 +82,14 @@ public:
     }
 
     Q_INVOKABLE QString get(QString name, int position = -1);
+    Q_INVOKABLE QColor getColor(QString name, int position = -1);
     Q_INVOKABLE double getDouble(QString name, int position = -1);
     Q_INVOKABLE QRectF getRect(QString name, int position = -1);
     Q_INVOKABLE void removeRectPercents(QString name);
     Q_INVOKABLE QStringList getGradient(QString name);
     Q_INVOKABLE void set(QString name, QString value, int position = -1);
+    Q_INVOKABLE void set(QString name, const QColor &value,
+                         int position = -1, mlt_keyframe_type keyframeType = mlt_keyframe_type(-1));
     Q_INVOKABLE void set(QString name, double value,
                          int position = -1, mlt_keyframe_type keyframeType = mlt_keyframe_type(-1));
     Q_INVOKABLE void set(QString name, int value,
@@ -105,9 +114,10 @@ public:
     /// returns the index of the new preset
     Q_INVOKABLE int  savePreset(const QStringList &propertyNames, const QString &name = QString());
     Q_INVOKABLE void deletePreset(const QString &name);
-    Q_INVOKABLE void analyze(bool isAudio = false);
+    Q_INVOKABLE void analyze(bool isAudio = false, bool deferJob = true);
     Q_INVOKABLE static int framesFromTime(const QString &time);
-    Q_INVOKABLE static QString timeFromFrames(int frames, TimeFormat format = TIME_TIMECODE_DF);
+    Q_INVOKABLE static QString timeFromFrames(int frames,
+                                              QmlFilter::TimeFormat format = TIME_TIMECODE_DF);
     Q_INVOKABLE void getHash();
     Mlt::Producer &producer()
     {
@@ -138,9 +148,12 @@ public:
     bool allowTrim() const;
     bool allowAnimateIn() const;
     bool allowAnimateOut() const;
+    Q_INVOKABLE void crop(const QRectF &rect);
+    void startUndoTracking(FilterController *controller, int row);
 
 public slots:
     void preset(const QString &name);
+    void updateChangeCommand(const QString &name);
 
 signals:
     void presetsChanged();
@@ -161,6 +174,8 @@ private:
     QString m_path;
     bool m_isNew;
     QStringList m_presets;
+    Filter::ChangeParameterCommand *m_changeCommand;
+    bool m_changeCommandPushed;
 
     QString objectNameOrService();
     int keyframeIndex(Mlt::Animation &animation, int position);
@@ -176,12 +191,11 @@ public slots:
     void onAnalyzeFinished(AbstractJob *job, bool isSuccess);
 
 private:
-    QString resultsFromXml(const QString &fileName, const QString &serviceName);
+    QString resultsFromXml(const QString &fileName);
     void updateFilter(Mlt::Filter &filter, const QString &results);
     void updateJob(EncodeJob *job, const QString &results);
 
     QUuid m_uuid;
-    QString m_serviceName;
 };
 
 #endif // FILTER_H
