@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2022 Meltytech, LLC
+ * Copyright (c) 2014-2023 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,6 +71,13 @@ public:
     static double GetSpeedFromProducer(Mlt::Producer *producer);
     static QString updateCaption(Mlt::Producer *producer);
     static void passProducerProperties(Mlt::Producer *src, Mlt::Producer *dst);
+    static bool warnIfLowDiskSpace(const QString &path);
+    static bool isFpsDifferent(double a, double b);
+    static QString getNextFile(const QString &filePath);
+    static QString trcString(int trc);
+    static bool trcIsCompatible(int trc);
+    static QString getConversionAdvice(Mlt::Producer *producer);
+    static mlt_color mltColorFromQColor(const QColor &color);
 };
 
 #endif // UTIL_H
