@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2022 Meltytech, LLC
+ * Copyright (c) 2012-2023 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,28 +34,13 @@
 #include "proxymanager.h"
 #include "dialogs/longuitask.h"
 #include "spatialmedia/spatialmedia.h"
+#include "transcoder.h"
+#include "jobs/bitrateviewerjob.h"
 
 #include <QtWidgets>
 
 static const auto kHandleSeconds = 15.0;
 
-DecodeTask::DecodeTask(AvformatProducerWidget *widget)
-    : QObject(0)
-    , QRunnable()
-    , m_frame(widget->producer()->get_frame())
-{
-    connect(this, SIGNAL(frameDecoded()), widget, SLOT(onFrameDecoded()));
-}
-
-void DecodeTask::run()
-{
-    mlt_image_format format = mlt_image_none;
-    int w = MLT.profile().width();
-    int h = MLT.profile().height();
-    m_frame->get_image(format, w, h);
-    emit frameDecoded();
-}
-
 AvformatProducerWidget::AvformatProducerWidget(QWidget *parent)
     : QWidget(parent)
     , ui(new Ui::AvformatProducerWidget)
@@ -67,11 +52,7 @@ AvformatProducerWidget::AvformatProducerWidget(QWidget *parent)
     ui->filenameLabel->setFrame(true);
     Util::setColorsToHighlight(ui->filenameLabel, QPalette::Base);
     connect(ui->applySpeedButton, SIGNAL(clicked()), SLOT(on_speedSpinBox_editingFinished()));
-    if (Settings.playerGPU())
-        connect(MLT.videoWidget(), SIGNAL(frameDisplayed(const SharedFrame &)), this,
-                SLOT(onFrameDisplayed(const SharedFrame &)));
-    else
-        connect(this, SIGNAL(producerChanged(Mlt::Producer *)), SLOT(onProducerChanged(Mlt::Producer *)));
+    connect(this, SIGNAL(producerChanged(Mlt::Producer *)), SLOT(onProducerChanged(Mlt::Producer *)));
 }
 
 AvformatProducerWidget::~AvformatProducerWidget()
@@ -155,7 +136,9 @@ void AvformatProducerWidget::offerConvert(QString message, bool set709Convert, b
                 m_producer->get_length() - 1;
     dialog.setSubClipChecked(setSubClip && (m_producer->get_in() > 0
                                             || m_producer->get_out() < m_producer->get_length() - 1));
-    convert(dialog);
+    Transcoder transcoder;
+    transcoder.addProducer(m_producer.data());
+    transcoder.convert(dialog);
 }
 
 void AvformatProducerWidget::keyPressEvent(QKeyEvent *event)
@@ -168,23 +151,17 @@ void AvformatProducerWidget::keyPressEvent(QKeyEvent *event)
     }
 }
 
-void AvformatProducerWidget::onFrameDisplayed(const SharedFrame &)
-{
-    // This forces avformat-novalidate or unloaded avformat to load and get
-    // media information.
-    delete m_producer->get_frame();
-    onFrameDecoded();
-    // We can stop listening to this signal if this is audio-only or if we have
-    // received the video resolution.
-    if (m_producer->get_int("audio_index") == -1 || m_producer->get_int("meta.media.width")
-            || m_producer->get_int("meta.media.height"))
-        disconnect(MLT.videoWidget(), SIGNAL(frameDisplayed(const SharedFrame &)), this, 0);
-}
-
 void AvformatProducerWidget::onProducerChanged(Mlt::Producer *producer)
 {
     if ( producer->get_producer() == m_producer->get_producer() ) {
-        QThreadPool::globalInstance()->start(new DecodeTask(this), 10);
+        if (Settings.playerGPU()) {
+            QTimer::singleShot(50, this, &AvformatProducerWidget::reloadProducerValues);
+        } else {
+            auto task = new ProbeTask(producer);
+            connect(task, &ProbeTask::probeFinished, this, &AvformatProducerWidget::reloadProducerValues,
+                    Qt::QueuedConnection);
+            QThreadPool::globalInstance()->start(task, 10);
+        }
     }
 }
 
@@ -231,11 +208,11 @@ void AvformatProducerWidget::reopen(Mlt::Producer *p)
             position = p->get_out();
         p->set("in", m_producer->get_in());
     }
+    MLT.stop();
     if (MLT.setProducer(p)) {
         AbstractProducerWidget::setProducer(0);
         return;
     }
-    MLT.stop();
     emit producerReopened(false);
     emit producerChanged(p);
     MLT.seek(position);
@@ -243,7 +220,7 @@ void AvformatProducerWidget::reopen(Mlt::Producer *p)
     setProducer(p);
 }
 
-void AvformatProducerWidget::recreateProducer()
+void AvformatProducerWidget::recreateProducer(bool getFrame)
 {
     Mlt::Producer *p = newProducer(MLT.profile());
     Util::passProducerProperties(m_producer.data(), p);
@@ -261,6 +238,11 @@ void AvformatProducerWidget::recreateProducer()
         out = qMin(qRound(out * speedRatio), length - 1);
         p->set("length", p->frames_to_time(length, mlt_time_clock));
         p->set_in_and_out(in, out);
+        if (getFrame) {
+            // Getting a frame updates some properties such as audio_index,
+            // which is used by AudioLevelsTask.
+            delete p->get_frame();
+        }
         emit producerChanged(p);
         delete p;
     } else {
@@ -268,8 +250,10 @@ void AvformatProducerWidget::recreateProducer()
     }
 }
 
-void AvformatProducerWidget::onFrameDecoded()
+void AvformatProducerWidget::reloadProducerValues()
 {
+    if (Settings.playerGPU())
+        m_producer->probe();
     int tabIndex = ui->tabWidget->currentIndex();
     ui->tabWidget->setTabEnabled(0, false);
     ui->tabWidget->setTabEnabled(1, false);
@@ -300,10 +284,21 @@ void AvformatProducerWidget::onFrameDecoded()
     }
     ui->rangeComboBox->setEnabled(true);
 
+    // Disable all actions if the file does not exist
+    auto exists = QFile::exists(resource);
+    ui->speedSpinBox->setEnabled(exists);
+    ui->speedComboBox->setEnabled(exists);
+    ui->applySpeedButton->setEnabled(exists);
+    ui->durationSpinBox->setEnabled(exists);
+    ui->menuButton->setEnabled(exists);
+    ui->convertButton->setEnabled(exists);
+    ui->reverseButton->setEnabled(exists);
+    ui->proxyButton->setEnabled(exists);
+
     // populate the track combos
     int n = m_producer->get_int("meta.media.nb_streams");
-    int videoIndex = 1;
-    int audioIndex = 1;
+    int videoIndex = 0;
+    int audioIndex = 0;
     int totalAudioChannels = 0;
     bool populateTrackCombos = (ui->videoTrackComboBox->count() == 0 &&
                                 ui->audioTrackComboBox->count() == 0);
@@ -320,16 +315,22 @@ void AvformatProducerWidget::onFrameDecoded()
             key = QString("meta.media.%1.codec.height").arg(i);
             QString height(m_producer->get(key.toLatin1().constData()));
             QString name = QString("%1: %2x%3 %4")
-                           .arg(videoIndex)
+                           .arg(videoIndex + 1)
                            .arg(width)
                            .arg(height)
                            .arg(codec);
             if (populateTrackCombos) {
                 if (ui->videoTrackComboBox->count() == 0)
                     ui->videoTrackComboBox->addItem(tr("None"), -1);
+#if LIBMLT_VERSION_INT >= ((7<<16)+(19<<8))
+                ui->videoTrackComboBox->addItem(name, videoIndex);
+            }
+            if (videoIndex == m_producer->get_int(kVideoIndexProperty)) {
+#else
                 ui->videoTrackComboBox->addItem(name, i);
             }
-            if (i == m_producer->get_int("video_index")) {
+            if (i == m_producer->get_int(kVideoIndexProperty)) {
+#endif
                 key = QString("meta.media.%1.codec.long_name").arg(i);
                 QString codec(m_producer->get(key.toLatin1().constData()));
                 ui->videoTableWidget->setItem(0, 1, new QTableWidgetItem(codec));
@@ -368,43 +369,11 @@ void AvformatProducerWidget::onFrameDecoded()
                 ui->videoTableWidget->setItem(4, 1, new QTableWidgetItem(csString));
                 key = QString("meta.media.%1.codec.color_trc").arg(i);
                 int trc = m_producer->get_int(key.toLatin1().constData());
-                QString trcString = tr("unknown (%1)").arg(trc);
-                switch (trc) {
-                case 0:
-                    trcString = tr("NA");
-                    break;
-                case 1:
-                    trcString = "ITU-R BT.709";
-                    break;
-                case 6:
-                    trcString = "ITU-R BT.601";
-                    break;
-                case 7:
-                    trcString = "SMPTE ST240";
-                    break;
-                case 11:
-                    trcString = "IEC 61966-2-4";
-                    break;
-                case 14:
-                    trcString = "ITU-R BT.2020";
-                    break;
-                case 15:
-                    trcString = "ITU-R BT.2020";
-                    break;
-                case 16:
-                    trcString = "SMPTE ST2084 (PQ)";
-                    break;
-                case 17:
-                    trcString = "SMPTE ST428";
-                    break;
-                case 18:
-                    trcString = "ARIB B67 (HLG)";
-                    break;
-                }
+                QString trcString = Util::trcString(trc);
                 QTableWidgetItem *trcItem = new QTableWidgetItem(trcString);
                 trcItem->setData(Qt::UserRole, QVariant(trc));
                 ui->videoTableWidget->setItem(5, 1, trcItem);
-                ui->videoTrackComboBox->setCurrentIndex(videoIndex);
+                ui->videoTrackComboBox->setCurrentIndex(videoIndex + 1);
             }
             ui->tabWidget->setTabEnabled(0, true);
             videoIndex++;
@@ -417,16 +386,22 @@ void AvformatProducerWidget::onFrameDecoded()
             key = QString("meta.media.%1.codec.sample_rate").arg(i);
             QString sampleRate(m_producer->get(key.toLatin1().constData()));
             QString name = QString("%1: %2 ch %3 KHz %4")
-                           .arg(audioIndex)
+                           .arg(audioIndex + 1)
                            .arg(channels)
                            .arg(sampleRate.toDouble() / 1000)
                            .arg(codec);
             if (populateTrackCombos) {
                 if (ui->audioTrackComboBox->count() == 0)
                     ui->audioTrackComboBox->addItem(tr("None"), -1);
+#if LIBMLT_VERSION_INT >= ((7<<16)+(19<<8))
+                ui->audioTrackComboBox->addItem(name, audioIndex);
+            }
+            if (QString::number(audioIndex) == m_producer->get(kAudioIndexProperty)) {
+#else
                 ui->audioTrackComboBox->addItem(name, i);
             }
-            if ( QString::number(i) == m_producer->get("audio_index")) {
+            if (QString::number(i) == m_producer->get(kAudioIndexProperty)) {
+#endif
                 key = QString("meta.media.%1.codec.long_name").arg(i);
                 QString codec(m_producer->get(key.toLatin1().constData()));
                 ui->audioTableWidget->setItem(0, 1, new QTableWidgetItem(codec));
@@ -437,7 +412,7 @@ void AvformatProducerWidget::onFrameDecoded()
                 key = QString("meta.media.%1.codec.sample_fmt").arg(i);
                 ui->audioTableWidget->setItem(3, 1, new QTableWidgetItem(
                                                   m_producer->get(key.toLatin1().constData())));
-                ui->audioTrackComboBox->setCurrentIndex(audioIndex);
+                ui->audioTrackComboBox->setCurrentIndex(audioIndex + 1);
             }
             ui->tabWidget->setTabEnabled(1, true);
             audioIndex++;
@@ -519,14 +494,9 @@ void AvformatProducerWidget::onFrameDecoded()
     ui->aspectDenSpinBox->setValue(dar_denominator);
     ui->aspectDenSpinBox->blockSignals(false);
 
-    double fps = m_producer->get_double("meta.media.frame_rate_num");
-    if (m_producer->get_double("meta.media.frame_rate_den") > 0)
-        fps /= m_producer->get_double("meta.media.frame_rate_den");
-    if (m_producer->get("force_fps"))
-        fps = m_producer->get_double("fps");
     bool isVariableFrameRate = m_producer->get_int("meta.media.variable_frame_rate");
-    if (fps != 0.0 ) {
-        ui->videoTableWidget->setItem(2, 1, new QTableWidgetItem(QString("%L1 %2").arg(fps, 0, 'f', 6)
+    if (fps() != 0.0 ) {
+        ui->videoTableWidget->setItem(2, 1, new QTableWidgetItem(QString("%L1 %2").arg(fps(), 0, 'f', 6)
                                                                  .arg(isVariableFrameRate ? tr("(variable)") : "")));
     }
 
@@ -564,37 +534,12 @@ void AvformatProducerWidget::onFrameDecoded()
     }
     ui->syncSlider->setValue(qRound(m_producer->get_double("video_delay") * 1000.0));
     setSyncVisibility();
-
-    if (Settings.showConvertClipDialog() && !m_producer->get_int(kShotcutSkipConvertProperty)) {
-        auto transferItem = ui->videoTableWidget->item(5, 1);
-        if (transferItem) LOG_INFO() << "color transfer" << transferItem->data(
-                                             Qt::UserRole).toInt() << "=" << transferItem->text();
-        if (transferItem && transferItem->data(Qt::UserRole).toInt() > 7
-                && transferItem->data(Qt::UserRole).toInt() != 11
-                && transferItem->data(Qt::UserRole).toInt() != 18) {
-            // Transfer characteristics > SMPTE240M Probably need conversion except IEC61966-2-4 is OK
-            QString trcString = ui->videoTableWidget->item(5, 1)->text();
-            LOG_INFO() << resource << "Probable HDR" << trcString;
-            offerConvert(
-                tr("This file uses color transfer characteristics %1, which may result in incorrect colors or brightness in Shotcut.").arg(
-                    trcString), true);
-        } else if (isVariableFrameRate) {
-            LOG_INFO() << resource << "is variable frame rate";
-            offerConvert(tr("This file is variable frame rate, which is not reliable for editing."));
-        } else if (QFile::exists(resource) && !MLT.isSeekable(m_producer.data())) {
-            LOG_INFO() << resource << "is not seekable";
-            offerConvert(tr("This file does not support seeking and cannot be used for editing."));
-        } else if (QFile::exists(resource) && resource.endsWith(".m2t")) {
-            LOG_INFO() << resource << "is HDV";
-            offerConvert(tr("This file format (HDV) is not reliable for editing."));
-        }
-    }
 }
 
 void AvformatProducerWidget::on_videoTrackComboBox_activated(int index)
 {
     if (m_producer) {
-        m_producer->set("video_index", ui->videoTrackComboBox->itemData(index).toInt());
+        m_producer->set(kVideoIndexProperty, ui->videoTrackComboBox->itemData(index).toInt());
         recreateProducer();
     }
 }
@@ -606,9 +551,9 @@ void AvformatProducerWidget::on_audioTrackComboBox_activated(int index)
         if (!m_producer->get(kDefaultAudioIndexProperty)) {
             m_producer->set(kDefaultAudioIndexProperty, m_producer->get_int("audio_index"));
         }
-        m_producer->set("audio_index",
+        m_producer->set(kAudioIndexProperty,
                         ui->audioTrackComboBox->itemData(index).toString().toUtf8().constData());
-        recreateProducer();
+        recreateProducer(true);
     }
 }
 
@@ -622,9 +567,6 @@ void AvformatProducerWidget::on_scanComboBox_activated(int index)
             // by setting them NULL.
             m_producer->set("force_progressive", QString::number(index).toLatin1().constData());
         emit producerChanged(producer());
-        if (Settings.playerGPU())
-            connect(MLT.videoWidget(), SIGNAL(frameDisplayed(const SharedFrame &)), this,
-                    SLOT(onFrameDisplayed(const SharedFrame &)));
     }
 }
 
@@ -635,9 +577,6 @@ void AvformatProducerWidget::on_fieldOrderComboBox_activated(int index)
         if (m_producer->get("force_tff") || tff != index)
             m_producer->set("force_tff", QString::number(index).toLatin1().constData());
         emit producerChanged(producer());
-        if (Settings.playerGPU())
-            connect(MLT.videoWidget(), SIGNAL(frameDisplayed(const SharedFrame &)), this,
-                    SLOT(onFrameDisplayed(const SharedFrame &)));
     }
 }
 
@@ -655,9 +594,6 @@ void AvformatProducerWidget::on_aspectNumSpinBox_valueChanged(int)
             m_producer->set(kAspectRatioDenominator, ui->aspectDenSpinBox->text().toLatin1().constData());
         }
         emit producerChanged(producer());
-        if (Settings.playerGPU())
-            connect(MLT.videoWidget(), SIGNAL(frameDisplayed(const SharedFrame &)), this,
-                    SLOT(onFrameDisplayed(const SharedFrame &)));
     }
 }
 
@@ -736,12 +672,14 @@ void AvformatProducerWidget::on_menuButton_clicked()
     if (ui->actionExportGPX->isEnabled()) {
         menu.addAction(ui->actionExportGPX);
     }
+    menu.addAction(ui->actionBitrateViewer);
     menu.exec(ui->menuButton->mapToGlobal(QPoint(0, 0)));
 }
 
 void AvformatProducerWidget::on_actionCopyFullFilePath_triggered()
 {
-    qApp->clipboard()->setText(Util::GetFilenameFromProducer(producer()));
+    auto s = Util::GetFilenameFromProducer(producer());
+    qApp->clipboard()->setText(QDir::toNativeSeparators(s));
 }
 
 void AvformatProducerWidget::on_notesTextEdit_textChanged()
@@ -788,205 +726,9 @@ void AvformatProducerWidget::on_actionFFmpegConvert_triggered()
     dialog.setWindowModality(QmlApplication::dialogModality());
     dialog.set709Convert(ui->videoTableWidget->item(5, 1)->data(Qt::UserRole).toInt() > 7);
     dialog.showSubClipCheckBox();
-    convert(dialog);
-}
-
-static QString getNextFile(const QString &filePath)
-{
-    QFileInfo info(filePath);
-    QString basename = info.completeBaseName();
-    QString extension = info.suffix();
-    if (extension.isEmpty()) {
-        extension = basename;
-        basename = QString();
-    }
-    for (unsigned i = 1; i < std::numeric_limits<unsigned>::max(); i++) {
-        QString filename = QString::fromLatin1("%1%2.%3").arg(basename).arg(i).arg(extension);
-        if (!info.dir().exists(filename))
-            return info.dir().filePath(filename);
-    }
-    return filePath;
-}
-
-void AvformatProducerWidget::convert(TranscodeDialog &dialog)
-{
-    int result = dialog.exec();
-    if (dialog.isCheckBoxChecked()) {
-        Settings.setShowConvertClipDialog(false);
-    }
-    if (result == QDialog::Accepted) {
-        QString resource = Util::GetFilenameFromProducer(producer());
-        QString path = Settings.savePath();
-        QStringList args;
-        QString nameFilter;
-        int in = -1;
-
-        args << "-loglevel" << "verbose";
-        args << "-i" << resource;
-        args << "-max_muxing_queue_size" << "9999";
-
-        if (dialog.isSubClip()) {
-            if (Settings.proxyEnabled()) {
-                m_producer->Mlt::Properties::clear(kOriginalResourceProperty);
-            }
-
-            // set trim options
-            if (m_producer->get(kFilterInProperty)) {
-                in = m_producer->get_int(kFilterInProperty);
-                int ss = qMax(0, in - qRound(m_producer->get_fps() * kHandleSeconds));
-                auto s = QString::fromLatin1(m_producer->frames_to_time(ss, mlt_time_clock));
-                args << "-ss" << s.replace(',', '.');
-                in -= ss;
-            } else {
-                args << "-ss" << QString::fromLatin1(m_producer->get_time("in", mlt_time_clock)).replace(',',
-                                                                                                         '.').replace(',', '.');
-            }
-            if (m_producer->get(kFilterOutProperty)) {
-                int out = m_producer->get_int(kFilterOutProperty);
-                int to = qMin(m_producer->get_playtime() - 1, out + qRound(m_producer->get_fps() * kHandleSeconds));
-                auto s = QString::fromLatin1(m_producer->frames_to_time(to, mlt_time_clock));
-                args << "-to" << s.replace(',', '.');
-            } else {
-                args << "-to" << QString::fromLatin1(m_producer->get_time("out", mlt_time_clock)).replace(',', '.');
-            }
-        }
-
-        // transcode all streams except data, subtitles, and attachments
-        auto audioIndex = m_producer->property_exists(kDefaultAudioIndexProperty) ? m_producer->get_int(
-                              kDefaultAudioIndexProperty) : m_producer->get_int("audio_index");
-        if (m_producer->get_int("video_index") < audioIndex) {
-            args << "-map" << "0:V?" << "-map" << "0:a?";
-        } else {
-            args << "-map" << "0:a?" << "-map" << "0:V?";
-        }
-        args << "-map_metadata" << "0" << "-ignore_unknown";
-
-        // Set Sample rate if different than source
-        if ( !dialog.sampleRate().isEmpty() ) {
-            args << "-ar" << dialog.sampleRate();
-        }
-
-        // Set video filters
-        args << "-vf";
-        QString filterString;
-        if (dialog.deinterlace()) {
-            QString deinterlaceFilter = QString("bwdif,");
-            filterString = filterString + deinterlaceFilter;
-        }
-        QString range;
-        if (ui->rangeComboBox->currentIndex())
-            range = "full";
-        else
-            range = "mpeg";
-        if (dialog.get709Convert()) {
-            QString convertFilter =
-                QString("zscale=t=linear:npl=100,format=gbrpf32le,zscale=p=bt709,tonemap=tonemap=hable:desat=0,zscale=t=bt709:m=bt709:r=tv,format=yuv422p,");
-            filterString = filterString + convertFilter;
-        }
-        filterString = filterString +
-                       QString("scale=flags=accurate_rnd+full_chroma_inp+full_chroma_int:in_range=%1:out_range=%2").arg(
-                           range).arg(range);
-        if (dialog.fpsOverride()) {
-            auto fps = QString("%1").arg(dialog.fps(), 0, 'f', 6);
-            int numerator, denominator;
-            Util::normalizeFrameRate(dialog.fps(), numerator, denominator);
-            if (denominator == 1001) {
-                fps = QString("%1/%2").arg(numerator).arg(denominator);
-            }
-            QString minterpFilter =
-                QString(",minterpolate='mi_mode=%1:mc_mode=aobmc:me_mode=bidir:vsbmc=1:fps=%2'").arg(
-                    dialog.frc()).arg(fps);
-            filterString = filterString + minterpFilter;
-        }
-        args << filterString;
-
-        // Specify color range
-        if (ui->rangeComboBox->currentIndex())
-            args << "-color_range" << "jpeg";
-        else
-            args << "-color_range" << "mpeg";
-
-        if (!dialog.deinterlace() && !ui->scanComboBox->currentIndex())
-            args << "-flags" << "+ildct+ilme" << "-top" << QString::number(
-                     ui->fieldOrderComboBox->currentIndex());
-
-        switch (dialog.format()) {
-        case 0:
-            path.append("/%1 - %2.mp4");
-            nameFilter = tr("MP4 (*.mp4);;All Files (*)");
-            args << "-f" << "mp4" << "-codec:a" << "ac3" << "-b:a" << "512k" << "-codec:v" << "libx264";
-            args << "-preset" << "medium" << "-g" << "1" << "-crf" << "15";
-            break;
-        case 1:
-            args << "-f" << "mov" << "-codec:a" << "pcm_f32le";
-            if (dialog.deinterlace() || ui->scanComboBox->currentIndex()) { // progressive
-                args << "-codec:v" << "dnxhd" << "-profile:v" << "dnxhr_hq" << "-pix_fmt" << "yuv422p";
-            } else { // interlaced
-                args << "-codec:v" << "prores_ks" << "-profile:v" << "standard";
-            }
-            path.append("/%1 - %2.mov");
-            nameFilter = tr("MOV (*.mov);;All Files (*)");
-            break;
-        case 2:
-            args << "-f" << "matroska" << "-codec:a" << "pcm_f32le" << "-codec:v" << "utvideo";
-            args << "-pix_fmt" << "yuv422p";
-            path.append("/%1 - %2.mkv");
-            nameFilter = tr("MKV (*.mkv);;All Files (*)");
-            break;
-        }
-        if (dialog.get709Convert()) {
-            args << "-colorspace" << "bt709" << "-color_primaries" << "bt709" << "-color_trc" << "bt709";
-        }
-        QFileInfo fi(resource);
-        QString suffix = dialog.isSubClip() ? tr("Sub-clip") + ' ' : tr("Converted");
-        path = path.arg(fi.completeBaseName(), suffix);
-        if (dialog.isSubClip()) {
-            path = getNextFile(path);
-        }
-        QString filename = QFileDialog::getSaveFileName(this, dialog.windowTitle(), path, nameFilter,
-                                                        nullptr, Util::getFileDialogOptions());
-        if (!filename.isEmpty()) {
-            if (filename == QDir::toNativeSeparators(resource)) {
-                QMessageBox::warning(this, dialog.windowTitle(),
-                                     QObject::tr("Unable to write file %1\n"
-                                                 "Perhaps you do not have permission.\n"
-                                                 "Try again with a different folder.")
-                                     .arg(fi.fileName()));
-                return;
-            }
-            if (Util::warnIfNotWritable(filename, this, dialog.windowTitle()))
-                return;
-
-            Settings.setSavePath(QFileInfo(filename).path());
-            args << "-y" << filename;
-            m_producer->Mlt::Properties::clear(kOriginalResourceProperty);
-
-            FfmpegJob *job = new FfmpegJob(filename, args, false);
-            job->setLabel(tr("Convert %1").arg(Util::baseName(filename)));
-            if (dialog.isSubClip()) {
-                if (m_producer->get(kMultitrackItemProperty)) {
-                    QString s = QString::fromLatin1(m_producer->get(kMultitrackItemProperty));
-                    QVector<QStringRef> parts = s.splitRef(':');
-                    if (parts.length() == 2) {
-                        int clipIndex = parts[0].toInt();
-                        int trackIndex = parts[1].toInt();
-                        QUuid uuid = MAIN.timelineClipUuid(trackIndex, clipIndex);
-                        if (!uuid.isNull()) {
-                            job->setPostJobAction(new ReplaceOnePostJobAction(resource, filename, QString(), uuid.toByteArray(),
-                                                                              in));
-                            JOBS.add(job);
-                        }
-                    }
-                } else {
-                    job->setPostJobAction(new OpenPostJobAction(resource, filename, QString()));
-                    JOBS.add(job);
-                }
-                return;
-            }
-            job->setPostJobAction(new ReplaceAllPostJobAction(resource, filename, Util::getHash(*m_producer)));
-            JOBS.add(job);
-        }
-    }
+    Transcoder transcoder;
+    transcoder.addProducer(m_producer.data());
+    transcoder.convert(dialog);
 }
 
 bool AvformatProducerWidget::revertToOriginalResource()
@@ -996,7 +738,7 @@ bool AvformatProducerWidget::revertToOriginalResource()
         m_producer->Mlt::Properties::clear(kOriginalResourceProperty);
         if (m_producer->get(kMultitrackItemProperty)) {
             QString s = QString::fromLatin1(m_producer->get(kMultitrackItemProperty));
-            QVector<QStringRef> parts = s.splitRef(':');
+            auto parts = s.split(':');
             if (parts.length() == 2) {
                 int clipIndex = parts[0].toInt();
                 int trackIndex = parts[1].toInt();
@@ -1032,6 +774,16 @@ void AvformatProducerWidget::setSyncVisibility()
     ui->syncSpinBox->setVisible(visible);
 }
 
+double AvformatProducerWidget::fps()
+{
+    double fps = m_producer->get_double("meta.media.frame_rate_num");
+    if (m_producer->get_double("meta.media.frame_rate_den") > 0)
+        fps /= m_producer->get_double("meta.media.frame_rate_den");
+    if (m_producer->get("force_fps"))
+        fps = m_producer->get_double("fps");
+    return fps;
+}
+
 void AvformatProducerWidget::on_reverseButton_clicked()
 {
     if (revertToOriginalResource())
@@ -1179,6 +931,11 @@ void AvformatProducerWidget::on_reverseButton_clicked()
             if (Util::warnIfNotWritable(filename, this, dialog.windowTitle()))
                 return;
 
+            if (Util::warnIfLowDiskSpace(filename)) {
+                MAIN.showStatusMessage(tr("Reverse canceled"));
+                return;
+            }
+
             Settings.setSavePath(QFileInfo(filename).path());
 
             // Make a temporary file name for the ffmpeg job.
@@ -1206,14 +963,14 @@ void AvformatProducerWidget::on_reverseButton_clicked()
 
             if (m_producer->get(kMultitrackItemProperty)) {
                 QString s = QString::fromLatin1(m_producer->get(kMultitrackItemProperty));
-                QVector<QStringRef> parts = s.splitRef(':');
+                auto parts = s.split(':');
                 if (parts.length() == 2) {
                     int clipIndex = parts[0].toInt();
                     int trackIndex = parts[1].toInt();
                     QUuid uuid = MAIN.timelineClipUuid(trackIndex, clipIndex);
                     if (!uuid.isNull()) {
                         meltJob->setPostJobAction(new ReplaceOnePostJobAction(resource, filename, tmpFileName,
-                                                                              uuid.toByteArray(), in));
+                                                                              uuid, in));
                         JOBS.add(meltJob);
                         return;
                     }
@@ -1439,7 +1196,7 @@ void AvformatProducerWidget::on_actionSetEquirectangular_triggered()
     QString filePath = QFileDialog::getSaveFileName(&MAIN, caption, directory, QString(),
                                                     nullptr, Util::getFileDialogOptions());
     if (!filePath.isEmpty()) {
-        if (SpatialMedia::injectSpherical(objectName().toStdString(), filePath.toStdString())) {
+        if (SpatialMedia::injectSpherical(info.filePath().toStdString(), filePath.toStdString())) {
             MAIN.showStatusMessage(tr("Successfully wrote %1").arg(QFileInfo(filePath).fileName()));
         } else {
             MAIN.showStatusMessage(tr("An error occurred saving the projection."));
@@ -1466,15 +1223,8 @@ void AvformatProducerWidget::on_actionFFmpegVideoQuality_triggered()
         args << "-filter_complex";
         int width = m_producer->get_int("meta.media.width");
         int height = m_producer->get_int("meta.media.height");
-        double fps = m_producer->get_double("meta.media.frame_rate_num");
         int frameRateNum, frameRateDen;
-        if (m_producer->get_double("meta.media.frame_rate_den") > 0) {
-            fps /= m_producer->get_double("meta.media.frame_rate_den");
-        }
-        if (m_producer->get("force_fps")) {
-            fps = m_producer->get_double("fps");
-        }
-        Util::normalizeFrameRate(fps, frameRateNum, frameRateDen);
+        Util::normalizeFrameRate(fps(), frameRateNum, frameRateDen);
         auto colorRange = (ui->rangeComboBox->currentIndex() == 1) ? "full" : "limited";
 
 #ifdef Q_OS_WIN
@@ -1486,7 +1236,7 @@ void AvformatProducerWidget::on_actionFFmpegVideoQuality_triggered()
         auto modelPath = (width < 3840
                           && height < 2160) ? dir.filePath("vmaf_v0.6.1.json") : dir.filePath("vmaf_4k_v0.6.1.json");
 #endif
-        args << QString("[0:v]fps=%4/%5,setpts=PTS-STARTPTS[reference];[1:v]scale=%1:%2:out_range=%6:flags=bicubic,fps=%4/%5,setpts=PTS-STARTPTS[distorted];[distorted][reference]libvmaf=log_fmt=csv:log_path='%8':feature='name=psnr|name=float_ssim':shortest=true:n_threads=%7:model='path=%3'")
+        args << QString("[0:v]scale=out_range=%6,fps=%4/%5,setpts=PTS-STARTPTS[reference];[1:v]scale=%1:%2:out_range=%6:flags=bicubic,fps=%4/%5,setpts=PTS-STARTPTS[distorted];[distorted][reference]libvmaf=log_fmt=csv:log_path='%8':feature='name=psnr|name=float_ssim':shortest=true:n_threads=%7:model='path=%3'")
              .arg(width).arg(height)
              .arg(modelPath)
              .arg(frameRateNum).arg(frameRateDen)
@@ -1519,3 +1269,40 @@ void AvformatProducerWidget::on_actionExportGPX_triggered()
     JOBS.add(new GoPro2GpxJob(resource, args));
 }
 
+void AvformatProducerWidget::on_speedComboBox_textActivated(const QString &arg1)
+{
+    if (arg1.isEmpty()) return;
+    ui->speedSpinBox->setValue(arg1.toDouble());
+    on_speedSpinBox_editingFinished();
+}
+
+ProbeTask::ProbeTask(Mlt::Producer *producer)
+    : QObject(0)
+    , QRunnable()
+    , m_producer(*producer)
+{
+}
+
+void ProbeTask::run()
+{
+    m_producer.probe();
+    emit probeFinished();
+}
+
+void AvformatProducerWidget::on_actionBitrateViewer_triggered()
+{
+    QStringList args;
+    args << "-v" << "quiet";
+    args << "-print_format" << "json=compact=1";
+    if (m_producer->get_int("video_index") >= 0)
+        args << "-select_streams" << QString::fromLatin1("V:%1").arg(m_producer->get_int(
+                                                                         kVideoIndexProperty));
+    else
+        args << "-select_streams" << QString::fromLatin1("a:%1").arg(m_producer->get_int(
+                                                                         kAudioIndexProperty));
+    args << "-show_entries" << "packet=size,duration_time,pts_time,flags";
+    args << Util::GetFilenameFromProducer(producer());
+    auto job = new BitrateViewerJob(args.last(), args, fps());
+    job->setLabel(tr("Bitrate %1").arg(Util::baseName(args.last())));
+    JOBS.add(job);
+}
